@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2003 Hiroyuki Yamamoto & The Sylpheed Claws Team
+ * Copyright (C) 1999-2004 Hiroyuki Yamamoto & The Sylpheed Claws Team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -317,20 +317,31 @@ static gchar *parse_action_cmd(gchar *action, MsgInfo *msginfo,
 static gboolean parse_append_filename(GString *cmd, MsgInfo *msginfo)
 {
 	gchar *filename;
+	gchar *p, *q;
+	gchar escape_ch[] = "\\ ";
 
 	g_return_val_if_fail(msginfo, FALSE);
 
 	filename = procmsg_get_message_file(msginfo);
 
-	if (filename) {
-		g_string_append(cmd, filename);
-		g_free(filename);
-	} else {
+	if (!filename) {
 		alertpanel_error(_("Could not get message file %d"),
 				 msginfo->msgnum);
 		return FALSE;
 	}
 
+	p = filename;
+	while ((q = strpbrk(p, "$\"`'\\ \t*?[]&|;<>()!#~")) != NULL) {
+		escape_ch[1] = *q;
+		*q = '\0';
+		g_string_append(cmd, p);
+		g_string_append(cmd, escape_ch);
+		p = q + 1;
+	}
+	g_string_append(cmd, p);
+
+	g_free(filename);
+
 	return TRUE;
 }
 
@@ -423,7 +434,7 @@ static void action_update_menu(GtkItemFactory *ifactory, gchar *branch_path,
 		action_p = strstr(action, ": ");
 		if (action_p && action_p[2] &&
 		    action_get_type(&action_p[2]) != ACTION_ERROR) {
-			action_p[0] = 0x00;
+			action_p[0] = '\0';
 			menu_path = g_strdup_printf("%s/%s", branch_path,
 						    action);
 			ifentry.path = menu_path;
@@ -721,6 +732,7 @@ static ChildInfo *fork_child(gchar *cmd, const gchar *msg_str,
 	}
 
 	debug_print("Forking child and grandchild.\n");
+	debug_print("Executing: /bin/sh -c %s\n", cmd);
 
 	pid = fork();
 	if (pid == 0) { /* Child */
@@ -760,7 +772,7 @@ static ChildInfo *fork_child(gchar *cmd, const gchar *msg_str,
 			cmdline[0] = "sh";
 			cmdline[1] = "-c";
 			cmdline[2] = cmd;
-			cmdline[3] = 0;
+			cmdline[3] = NULL;
 			execvp("/bin/sh", cmdline);
 
 			perror("execvp");
