@@ -122,34 +122,6 @@ gchar *addrbook_get_name( AddressBookFile *book ) {
 	return book->name;
 }
 
-static gint addrcache_free_item_vis( gpointer key, gpointer value, gpointer data ) {
-	AddrItemObject *obj = ( AddrItemObject * ) value;
-
-	if( ADDRITEM_TYPE(obj) == ITEMTYPE_PERSON ) {
-		addritem_free_item_person( ( ItemPerson * ) obj );
-	}
-	else if( ADDRITEM_TYPE(obj) == ITEMTYPE_GROUP ) {
-		addritem_free_item_group( ( ItemGroup * ) obj );
-	}
-	else if( ADDRITEM_TYPE(obj) == ITEMTYPE_FOLDER ) {
-		addritem_free_item_folder( ( ItemFolder * ) obj );
-	}
-	key = NULL;
-	value = NULL;
-	return 0;
-}
-
-/*
-* Free hash table of address book items.
-*/
-static void addrcache_free_item_hash( GHashTable *table ) {
-	g_return_if_fail( table != NULL );
-	g_hash_table_freeze( table );
-	g_hash_table_foreach_remove( table, addrcache_free_item_vis, NULL );
-	g_hash_table_thaw( table );
-	g_hash_table_destroy( table );
-}
-
 /*
 * Empty address book.
 */
@@ -1360,6 +1332,7 @@ ItemPerson *addrbook_add_address_list( AddressBookFile *book, ItemFolder *folder
 	return person;
 }
 
+#if 0
 /*
 * Load hash table visitor function.
 */
@@ -1384,6 +1357,7 @@ static void addrbook_load_hash_table_email( AddressBookFile *book, GHashTable *t
 	g_return_if_fail( table != NULL );
 	g_hash_table_foreach( book->addressCache->itemHash, addrbook_load_hash_table_email_vis, table );
 }
+#endif
 
 /*
 * Build available email list visitor function.
@@ -1756,6 +1730,7 @@ gchar *addrbook_gen_new_file_name( gint fileNum ) {
 * ***********************************************************************
 */
 
+#if 0
 static void addrbook_show_attribs( GList *attr ) {
 	while( attr ) {
 		gchar *name = ((XMLAttr *)attr->data)->name;
@@ -1765,6 +1740,7 @@ static void addrbook_show_attribs( GList *attr ) {
 	}
 	printf( "\t---\n" );
 }
+#endif
 
 /*
 * Test email address list.
