@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2001 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2004 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,6 +33,14 @@ typedef enum
 	G_ALERTDISABLE	= 1 << 16
 } AlertValue;
 
+typedef enum
+{
+	ALERT_NOTICE,
+	ALERT_QUESTION,
+	ALERT_WARNING,
+	ALERT_ERROR
+} AlertType;
+
 #define G_ALERT_VALUE_MASK	0x0000ffff
 
 AlertValue alertpanel	(const gchar	*title,
@@ -42,10 +50,12 @@ AlertValue alertpanel	(const gchar	*title,
 			 const gchar	*button3_label);
 
 void alertpanel_message	(const gchar	*title,
-			 const gchar	*message);
+			 const gchar	*message,
+			 AlertType	 type);
 
 AlertValue alertpanel_message_with_disable	(const gchar	*title,
-						 const gchar	*message);
+						 const gchar	*message,
+						 AlertType	 type);
 
 void alertpanel_notice	(const gchar	*format,
 			 ...) G_GNUC_PRINTF(1, 2);
