@@ -44,6 +44,8 @@
 #define COMMON_RC		"sylpheedrc"
 #define ACCOUNT_RC		"accountrc"
 #define FILTER_RC		"filterrc"
+#define FILTER_LIST		"filter.xml"
+#define FILTER_HEADER_RC	"filterheaderrc"
 #define CUSTOM_HEADER_RC	"customheaderrc"
 #define DISPLAY_HEADER_RC	"dispheaderrc"
 #define MENU_RC			"menurc"
@@ -68,7 +70,7 @@
 /* #define DEFAULT_INC_PATH	"/usr/bin/imget" */
 /* #define DEFAULT_INC_PROGRAM	"imget" */
 #define DEFAULT_SENDMAIL_CMD	"/usr/sbin/sendmail -t -i"
-#define DEFAULT_BROWSER_CMD	"mozilla -remote 'openURL(%s, new-window)'"
+#define DEFAULT_BROWSER_CMD	"mozilla-firefox -remote 'openURL(%s,new-window)'"
 
 #ifdef _PATH_MAILDIR
 #  define DEFAULT_SPOOL_PATH	_PATH_MAILDIR
