@@ -166,6 +166,7 @@ FolderItem *folder_item_new(const gchar *name, const gchar *path)
 	item->new = 0;
 	item->unread = 0;
 	item->total = 0;
+	item->unmarked_num = 0;
 	item->last_num = -1;
 	item->no_sub = FALSE;
 	item->no_select = FALSE;
@@ -550,6 +551,20 @@ FolderItem *folder_find_item_from_path(const gchar *path)
 	return d[1];
 }
 
+FolderItem *folder_find_child_item_by_name(FolderItem *item, const gchar *name)
+{
+	GNode *node;
+	FolderItem *child;
+
+	for (node = item->node->children; node != NULL; node = node->next) {
+		child = FOLDER_ITEM(node->data);
+		if (strcmp2(g_basename(child->path), name) == 0)
+			return child;
+	}
+
+	return NULL;
+}
+
 static const struct {
 	gchar *str;
 	FolderType type;
@@ -883,6 +898,52 @@ gchar *folder_item_fetch_msg(FolderItem *item, gint num)
 	return folder->klass->fetch_msg(folder, item, num);
 }
 
+gint folder_item_fetch_all_msg(FolderItem *item)
+{
+	Folder *folder;
+	GSList *mlist;
+	GSList *cur;
+	gint num = 0;
+	gint ret = 0;
+
+	g_return_val_if_fail(item != NULL, -1);
+
+	debug_print("fetching all messages in %s ...\n", item->path);
+
+	folder = item->folder;
+
+	if (folder->ui_func)
+		folder->ui_func(folder, item, folder->ui_func_data ?
+				folder->ui_func_data : GINT_TO_POINTER(num));
+
+	mlist = folder_item_get_msg_list(item, TRUE);
+
+	for (cur = mlist; cur != NULL; cur = cur->next) {
+		MsgInfo *msginfo = (MsgInfo *)cur->data;
+		gchar *msg;
+
+		num++;
+		if (folder->ui_func)
+			folder->ui_func(folder, item,
+					folder->ui_func_data ?
+					folder->ui_func_data :
+					GINT_TO_POINTER(num));
+
+		msg = folder_item_fetch_msg(item, msginfo->msgnum);
+		if (!msg) {
+			g_warning("Can't fetch message %d. Aborting.\n",
+				  msginfo->msgnum);
+			ret = -1;
+			break;
+		}
+		g_free(msg);
+	}
+
+	procmsg_msg_list_free(mlist);
+
+	return ret;
+}
+
 MsgInfo *folder_item_get_msginfo(FolderItem *item, gint num)
 {
 	Folder *folder;
@@ -978,18 +1039,13 @@ gint folder_item_copy_msgs(FolderItem *dest, GSList *msglist)
 gint folder_item_remove_msg(FolderItem *item, MsgInfo *msginfo)
 {
 	Folder *folder;
-	gint ret;
 
 	g_return_val_if_fail(item != NULL, -1);
 	g_return_val_if_fail(item->folder->klass->remove_msg != NULL, -1);
 
 	folder = item->folder;
 
-	ret = folder->klass->remove_msg(folder, item, msginfo);
-	if (ret == 0 && msginfo->msgnum == item->last_num)
-		folder->klass->scan(folder, item);
-
-	return ret;
+	return folder->klass->remove_msg(folder, item, msginfo);
 }
 
 gint folder_item_remove_msgs(FolderItem *item, GSList *msglist)
@@ -1001,10 +1057,7 @@ gint folder_item_remove_msgs(FolderItem *item, GSList *msglist)
 
 	folder = item->folder;
 	if (folder->klass->remove_msgs) {
-		ret = folder->klass->remove_msgs(folder, item, msglist);
-		if (ret == 0)
-			folder->klass->scan(folder, item);
-		return ret;
+		return folder->klass->remove_msgs(folder, item, msglist);
 	}
 
 	while (msglist != NULL) {
