@@ -163,6 +163,9 @@ static void folderview_rename_folder_cb	(FolderView	*folderview,
 static void folderview_delete_folder_cb	(FolderView	*folderview,
 					 guint		 action,
 					 GtkWidget	*widget);
+static void folderview_empty_trash_cb	(FolderView	*folderview,
+					 guint		 action,
+					 GtkWidget	*widget);
 static void folderview_remove_mailbox_cb(FolderView	*folderview,
 					 guint		 action,
 					 GtkWidget	*widget);
@@ -214,12 +217,12 @@ static GtkItemFactoryEntry folderview_mail_popup_entries[] =
 	{N_("/_Rename folder..."),	NULL, folderview_rename_folder_cb, 0, NULL},
 	{N_("/_Delete folder"),		NULL, folderview_delete_folder_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
+	{N_("/Empty _trash"),		NULL, folderview_empty_trash_cb, 0, NULL},
+	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Check for new messages"),
 					NULL, folderview_update_tree_cb, 0, NULL},
 	{N_("/R_ebuild folder tree"),	NULL, folderview_update_tree_cb, 1, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
-	{N_("/Remove _mailbox"),	NULL, folderview_remove_mailbox_cb, 0, NULL},
-	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Search messages..."),	NULL, folderview_search_cb, 0, NULL},
 	{N_("/_Properties..."),		NULL, folderview_property_cb, 0, NULL}
 };
@@ -230,14 +233,14 @@ static GtkItemFactoryEntry folderview_imap_popup_entries[] =
 	{N_("/_Rename folder..."),	NULL, folderview_rename_folder_cb, 0, NULL},
 	{N_("/_Delete folder"),		NULL, folderview_delete_folder_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
+	{N_("/Empty _trash"),		NULL, folderview_empty_trash_cb, 0, NULL},
+	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/Down_load"),		NULL, folderview_download_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Check for new messages"),
 					NULL, folderview_update_tree_cb, 0, NULL},
 	{N_("/R_ebuild folder tree"),	NULL, folderview_update_tree_cb, 1, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
-	{N_("/Remove _IMAP4 account"),	NULL, folderview_rm_imap_server_cb, 0, NULL},
-	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Search messages..."),	NULL, folderview_search_cb, 0, NULL},
 	{N_("/_Properties..."),		NULL, folderview_property_cb, 0, NULL}
 };
@@ -253,8 +256,6 @@ static GtkItemFactoryEntry folderview_news_popup_entries[] =
 	{N_("/_Check for new messages"),
 					NULL, folderview_update_tree_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
-	{N_("/Remove _news account"),	NULL, folderview_rm_news_server_cb, 0, NULL},
-	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Search messages..."),	NULL, folderview_search_cb, 0, NULL},
 	{N_("/_Properties..."),		NULL, folderview_property_cb, 0, NULL}
 };
@@ -541,6 +542,14 @@ void folderview_select_next_unread(FolderView *folderview)
 		folderview_select_node(folderview, node);
 }
 
+FolderItem *folderview_get_selected_item(FolderView *folderview)
+{
+	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
+
+	if (!folderview->selected) return NULL;
+	return gtk_ctree_node_get_row_data(ctree, folderview->selected);
+}
+
 void folderview_update_msg_num(FolderView *folderview, GtkCTreeNode *row)
 {
 	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
@@ -685,7 +694,7 @@ static GtkWidget *label_window_create(const gchar *str)
 	return window;
 }
 
-void folderview_rescan_tree(Folder *folder)
+static void folderview_rescan_tree(FolderView *folderview, Folder *folder)
 {
 	GtkWidget *window;
 	AlertValue avalue;
@@ -701,12 +710,14 @@ void folderview_rescan_tree(Folder *folder)
 	if (avalue != G_ALERTDEFAULT) return;
 
 	if (!FOLDER_IS_LOCAL(folder) &&
-	    !main_window_toggle_online_if_offline(main_window_get()))
+	    !main_window_toggle_online_if_offline(folderview->mainwin))
 		return;
 
 	inc_lock();
 	window = label_window_create(_("Rebuilding folder tree..."));
 
+	summary_show(folderview->summaryview, NULL, FALSE);
+
 	folder_set_ui_func(folder, folderview_scan_tree_func, NULL);
 	if (folder->klass->scan_tree(folder) < 0)
 		alertpanel_error(_("Rebuilding of the folder tree failed."));
@@ -1244,9 +1255,19 @@ void folderview_new_folder(FolderView *folderview)
 	g_return_if_fail(item != NULL);
 	g_return_if_fail(item->folder != NULL);
 
-	if (FOLDER_IS_LOCAL(item->folder) ||
-	    FOLDER_TYPE(item->folder) == F_IMAP)
+	switch (FOLDER_TYPE(item->folder)) {
+	case F_MH:
+	case F_MBOX:
+	case F_MAILDIR:
+	case F_IMAP:
 		folderview_new_folder_cb(folderview, 0, NULL);
+		break;
+	case F_NEWS:
+		folderview_new_news_group_cb(folderview, 0, NULL);
+		break;
+	default:
+		break;
+	}
 }
 
 void folderview_rename_folder(FolderView *folderview)
@@ -1296,11 +1317,72 @@ void folderview_delete_folder(FolderView *folderview)
 		folderview_delete_folder_cb(folderview, 0, NULL);
 		break;
 	case F_NEWS:
+		folderview_rm_news_group_cb(folderview, 0, NULL);
+		break;
 	default:
 		break;
 	}
 }
 
+void folderview_check_new_selected(FolderView *folderview)
+{
+	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
+	FolderItem *item;
+
+	if (!folderview->selected) return;
+
+	item = gtk_ctree_node_get_row_data(ctree, folderview->selected);
+	g_return_if_fail(item != NULL);
+	g_return_if_fail(item->folder != NULL);
+	if (item->parent != NULL) return;
+
+	folderview_check_new(item->folder);
+}
+
+void folderview_remove_mailbox(FolderView *folderview)
+{
+	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
+	FolderItem *item;
+
+	if (!folderview->selected) return;
+
+	item = gtk_ctree_node_get_row_data(ctree, folderview->selected);
+	g_return_if_fail(item != NULL);
+	g_return_if_fail(item->folder != NULL);
+	if (item->parent != NULL) return;
+
+	switch (FOLDER_TYPE(item->folder)) {
+	case F_MH:
+	case F_MBOX:
+	case F_MAILDIR:
+		folderview_remove_mailbox_cb(folderview, 0, NULL);
+		break;
+	case F_IMAP:
+		folderview_rm_imap_server_cb(folderview, 0, NULL);
+		break;
+	case F_NEWS:
+		folderview_rm_news_server_cb(folderview, 0, NULL);
+		break;
+	default:
+		break;
+	}
+}
+
+void folderview_rebuild_tree(FolderView *folderview)
+{
+	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
+	FolderItem *item;
+
+	if (!folderview->selected) return;
+
+	item = gtk_ctree_node_get_row_data(ctree, folderview->selected);
+	g_return_if_fail(item != NULL);
+	g_return_if_fail(item->folder != NULL);
+	if (item->parent != NULL) return;
+
+	folderview_rescan_tree(folderview, item->folder);
+}
+
 
 /* callback functions */
 
@@ -1315,6 +1397,7 @@ static void folderview_button_pressed(GtkWidget *ctree, GdkEventButton *event,
 	gboolean new_folder      = FALSE;
 	gboolean rename_folder   = FALSE;
 	gboolean delete_folder   = FALSE;
+	gboolean empty_trash     = FALSE;
 	gboolean download_msg    = FALSE;
 	gboolean update_tree     = FALSE;
 	gboolean rescan_tree     = FALSE;
@@ -1378,6 +1461,8 @@ static void folderview_button_pressed(GtkWidget *ctree, GdkEventButton *event,
 				update_tree = rescan_tree = TRUE;
 			else if (item->stype == F_NORMAL)
 				rename_folder = delete_folder = TRUE;
+			else if (item->stype == F_TRASH)
+				empty_trash = TRUE;
 		} else if (FOLDER_TYPE(folder) == F_NEWS) {
 			if (item->parent != NULL)
 				delete_folder = TRUE;
@@ -1398,9 +1483,9 @@ static void folderview_button_pressed(GtkWidget *ctree, GdkEventButton *event,
 		SET_SENS(mail_factory, "/Create new folder...", new_folder);
 		SET_SENS(mail_factory, "/Rename folder...", rename_folder);
 		SET_SENS(mail_factory, "/Delete folder", delete_folder);
+		SET_SENS(mail_factory, "/Empty trash", empty_trash);
 		SET_SENS(mail_factory, "/Check for new messages", update_tree);
 		SET_SENS(mail_factory, "/Rebuild folder tree", rescan_tree);
-		SET_SENS(mail_factory, "/Remove mailbox", remove_tree);
 		SET_SENS(mail_factory, "/Search messages...", search_folder);
 		SET_SENS(mail_factory, "/Properties...", folder_property);
 	} else if (FOLDER_TYPE(folder) == F_IMAP) {
@@ -1409,10 +1494,10 @@ static void folderview_button_pressed(GtkWidget *ctree, GdkEventButton *event,
 		SET_SENS(imap_factory, "/Create new folder...", new_folder);
 		SET_SENS(imap_factory, "/Rename folder...", rename_folder);
 		SET_SENS(imap_factory, "/Delete folder", delete_folder);
+		SET_SENS(imap_factory, "/Empty trash", empty_trash);
 		SET_SENS(imap_factory, "/Download", download_msg);
 		SET_SENS(imap_factory, "/Check for new messages", update_tree);
 		SET_SENS(imap_factory, "/Rebuild folder tree", rescan_tree);
-		SET_SENS(imap_factory, "/Remove IMAP4 account", remove_tree);
 		SET_SENS(imap_factory, "/Search messages...", search_folder);
 		SET_SENS(imap_factory, "/Properties...", folder_property);
 	} else if (FOLDER_TYPE(folder) == F_NEWS) {
@@ -1422,7 +1507,6 @@ static void folderview_button_pressed(GtkWidget *ctree, GdkEventButton *event,
 		SET_SENS(news_factory, "/Remove newsgroup", delete_folder);
 		SET_SENS(news_factory, "/Download", download_msg);
 		SET_SENS(news_factory, "/Check for new messages", update_tree);
-		SET_SENS(news_factory, "/Remove news account", remove_tree);
 		SET_SENS(news_factory, "/Search messages...", search_folder);
 		SET_SENS(news_factory, "/Properties...", folder_property);
 	} else
@@ -1484,6 +1568,8 @@ static void folderview_selected(GtkCTree *ctree, GtkCTreeNode *row,
 
 	folderview->selected = row;
 
+	main_window_set_menu_sensitive(folderview->mainwin);
+
 	if (folderview->opened == row) {
 		folderview->open_folder = FALSE;
 		return;
@@ -1654,8 +1740,6 @@ static void folderview_update_tree_cb(FolderView *folderview, guint action,
 
 	if (!folderview->selected) return;
 
-	summary_show(folderview->summaryview, NULL, FALSE);
-
 	item = gtk_ctree_node_get_row_data(ctree, folderview->selected);
 	g_return_if_fail(item != NULL);
 	g_return_if_fail(item->folder != NULL);
@@ -1663,7 +1747,7 @@ static void folderview_update_tree_cb(FolderView *folderview, guint action,
 	if (action == 0)
 		folderview_check_new(item->folder);
 	else
-		folderview_rescan_tree(item->folder);
+		folderview_rescan_tree(folderview, item->folder);
 }
 
 static void folderview_new_folder_cb(FolderView *folderview, guint action,
@@ -1859,6 +1943,37 @@ static void folderview_delete_folder_cb(FolderView *folderview, guint action,
 	folder_write_list();
 }
 
+static void folderview_empty_trash_cb(FolderView *folderview, guint action,
+				      GtkWidget *widget)
+{
+	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
+	FolderItem *item;
+	Folder *folder;
+
+	if (!folderview->selected) return;
+
+	item = gtk_ctree_node_get_row_data(ctree, folderview->selected);
+	g_return_if_fail(item != NULL);
+	g_return_if_fail(item->path != NULL);
+	g_return_if_fail(item->folder != NULL);
+
+	folder = item->folder;
+
+	if (folder->trash != item) return;
+	if (item->stype != F_TRASH) return;
+
+	if (alertpanel(_("Empty trash"), _("Empty all messages in trash?"),
+		       _("Yes"), _("No"), NULL) != G_ALERTDEFAULT)
+		return;
+
+	procmsg_empty_trash(folder->trash);
+	statusbar_pop_all();
+	folderview_update_item(folder->trash, TRUE);
+
+	if (folderview->opened == folderview->selected)
+		gtk_widget_grab_focus(folderview->ctree);
+}
+
 static void folderview_remove_mailbox_cb(FolderView *folderview, guint action,
 					 GtkWidget *widget)
 {
@@ -2036,7 +2151,7 @@ static void folderview_rm_news_group_cb(FolderView *folderview, guint action,
 	g_return_if_fail(FOLDER_TYPE(item->folder) == F_NEWS);
 	g_return_if_fail(item->folder->account != NULL);
 
-	name = trim_string(item->path, 32);
+	name = trim_string_before(item->path, 32);
 	message = g_strdup_printf(_("Really delete newsgroup `%s'?"), name);
 	avalue = alertpanel(_("Delete newsgroup"), message,
 			    _("Yes"), _("+No"), NULL);
