@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2003 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2004 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,13 +33,25 @@
 #include <gtk/gtkctree.h>
 #include <gtk/gtkcombo.h>
 #include <gtk/gtktext.h>
+#include <gtk/gtkitemfactory.h>
 #include <stdlib.h>
 #if HAVE_WCHAR_H
 #  include <wchar.h>
 #endif
 
+typedef struct _ComboButton	ComboButton;
+
 #include "gtkstext.h"
 
+struct _ComboButton
+{
+	GtkWidget *arrow;
+	GtkWidget *button;
+	GtkWidget *menu;
+	GtkItemFactory *factory;
+	gpointer data;
+};
+
 #define GTK_EVENTS_FLUSH() \
 { \
 	while (gtk_events_pending()) \
@@ -86,6 +98,12 @@ void gtkut_button_set_create		(GtkWidget	**bbox,
 					 GtkWidget	**button3,
 					 const gchar	 *label3);
 
+ComboButton *gtkut_combo_button_create	(GtkWidget		*button,
+					 GtkItemFactoryEntry	*entries,
+					 gint			 n_entries,
+					 const gchar		*path,
+					 gpointer		 data);
+
 void gtkut_ctree_node_move_if_on_the_edge
 					(GtkCTree	*ctree,
 					 GtkCTreeNode	*node);
@@ -112,6 +130,7 @@ void gtkut_combo_set_items		(GtkCombo	*combo,
 					 const gchar	*str1, ...);
 
 gchar *gtkut_editable_get_selection	(GtkEditable	*editable);
+void gtkut_editable_disable_im		(GtkEditable	*editable);
 
 void gtkut_container_remove		(GtkContainer	*container,
 					 GtkWidget	*widget);
