@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2002 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2004 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -40,9 +40,19 @@
 #include "imageview.h"
 #include "utils.h"
 
-static void get_resized_size(gint w, gint h, gint aw, gint ah,
-			     gint *sw, gint *sh);
+static void get_resized_size	(gint	 w,
+				 gint	 h,
+				 gint	 aw,
+				 gint	 ah,
+				 gint	*sw,
+				 gint	*sh);
 
+static gint button_press_cb	(GtkWidget	*widget,
+				 GdkEventButton	*event,
+				 gpointer	 data);
+static void size_allocate_cb	(GtkWidget	*widget,
+				 GtkAllocation	*allocation,
+				 gpointer	 data);
 
 ImageView *imageview_create(void)
 {
@@ -58,16 +68,25 @@ ImageView *imageview_create(void)
 				       GTK_POLICY_AUTOMATIC);
 	gtk_widget_set_usize(scrolledwin, prefs_common.mainview_width, -1);
 
+	gtk_signal_connect(GTK_OBJECT(scrolledwin), "button_press_event",
+			   GTK_SIGNAL_FUNC(button_press_cb), imageview);
+	gtk_signal_connect(GTK_OBJECT(scrolledwin), "size_allocate",
+			   GTK_SIGNAL_FUNC(size_allocate_cb), imageview);
+
 	gtk_widget_show_all(scrolledwin);
 
 	imageview->scrolledwin  = scrolledwin;
 	imageview->image        = NULL;
+	imageview->resize       = FALSE;
 
 	return imageview;
 }
 
 void imageview_init(ImageView *imageview)
 {
+#if HAVE_GDK_IMLIB
+	gdk_imlib_set_cache_info(0, 0);
+#endif
 }
 
 #if HAVE_GDK_PIXBUF
@@ -75,7 +94,6 @@ void imageview_show_image(ImageView *imageview, MimeInfo *mimeinfo,
 			  const gchar *file, gboolean resize)
 {
 	GdkPixbuf *pixbuf;
-	GdkPixbuf *pixbuf_scaled;
 	GdkPixmap *pixmap;
 	GdkBitmap *mask;
 	gint avail_width;
@@ -85,19 +103,24 @@ void imageview_show_image(ImageView *imageview, MimeInfo *mimeinfo,
 
 	g_return_if_fail(imageview != NULL);
 
-	imageview_clear(imageview);
-
-	pixbuf = gdk_pixbuf_new_from_file(file);
+	if (file) {
+		imageview_clear(imageview);
+		pixbuf = gdk_pixbuf_new_from_file(file);
+		imageview->image_data = pixbuf;
+	} else {
+		pixbuf = (GdkPixbuf *)imageview->image_data;
+	}
 
 	if (!pixbuf) {
 		g_warning(_("Can't load the image."));	
 		return;
 	}
 
-	if (imageview->messageview->mainwin)
-		main_window_cursor_wait(imageview->messageview->mainwin);
+	imageview->resize = resize;
 
 	if (resize) {
+		GdkPixbuf *pixbuf_scaled;
+
 		avail_width = imageview->scrolledwin->parent->allocation.width;
 		avail_height = imageview->scrolledwin->parent->allocation.height;
 		if (avail_width > 8) avail_width -= 8;
@@ -110,11 +133,11 @@ void imageview_show_image(ImageView *imageview, MimeInfo *mimeinfo,
 
 		pixbuf_scaled = gdk_pixbuf_scale_simple
 			(pixbuf, new_width, new_height, GDK_INTERP_BILINEAR);
-		gdk_pixbuf_unref(pixbuf);
-		pixbuf = pixbuf_scaled;
-	}
-
-	gdk_pixbuf_render_pixmap_and_mask(pixbuf, &pixmap, &mask, 0);
+		gdk_pixbuf_render_pixmap_and_mask(pixbuf_scaled,
+						  &pixmap, &mask, 0);
+		gdk_pixbuf_unref(pixbuf_scaled);
+	} else
+		gdk_pixbuf_render_pixmap_and_mask(pixbuf, &pixmap, &mask, 0);
 
 	if (!imageview->image) {
 		imageview->image = gtk_pixmap_new(pixmap, mask);
@@ -125,12 +148,9 @@ void imageview_show_image(ImageView *imageview, MimeInfo *mimeinfo,
 	} else
 		gtk_pixmap_set(GTK_PIXMAP(imageview->image), pixmap, mask);
 
+	gdk_pixmap_unref(pixmap);
+
 	gtk_widget_show(imageview->image);
-
-	gdk_pixbuf_unref(pixbuf);
-
-	if (imageview->messageview->mainwin)
-		main_window_cursor_normal(imageview->messageview->mainwin);
 }
 #else
 #if HAVE_GDK_IMLIB
@@ -138,6 +158,8 @@ void imageview_show_image(ImageView *imageview, MimeInfo *mimeinfo,
 			  const gchar *file, gboolean resize)
 {
 	GdkImlibImage *im;
+	GdkPixmap *pixmap;
+	GdkBitmap *mask;
 	gint avail_width;
 	gint avail_height;
 	gint new_width;
@@ -145,17 +167,20 @@ void imageview_show_image(ImageView *imageview, MimeInfo *mimeinfo,
 
 	g_return_if_fail(imageview != NULL);
 
-	imageview_clear(imageview);
-
-	im = gdk_imlib_load_image((gchar *)file);
+	if (file) {
+		imageview_clear(imageview);
+		im = gdk_imlib_load_image((gchar *)file);
+		imageview->image_data = im;
+	} else {
+		im = (GdkImlibImage *)imageview->image_data;
+	}
 
 	if (!im) {
 		g_warning(_("Can't load the image."));	
 		return;
 	}
 
-	if (imageview->messageview->mainwin)
-		main_window_cursor_wait(imageview->messageview->mainwin);
+	imageview->resize = resize;
 
 	if (resize) {
 		avail_width = imageview->scrolledwin->parent->allocation.width;
@@ -166,31 +191,25 @@ void imageview_show_image(ImageView *imageview, MimeInfo *mimeinfo,
 		get_resized_size(im->rgb_width, im->rgb_height,
 				 avail_width, avail_height,
 				 &new_width, &new_height);
-	} else {
-		new_width = im->rgb_width;
-		new_height = im->rgb_height;
-	}
+		gdk_imlib_render(im, new_width, new_height);
+	} else
+		gdk_imlib_render(im, im->rgb_width, im->rgb_height);
 
-	gdk_imlib_render(im, new_width, new_height);
+	pixmap = gdk_imlib_move_image(im);
+	mask = gdk_imlib_move_mask(im);
 
 	if (!imageview->image) {
-		imageview->image = gtk_pixmap_new(gdk_imlib_move_image(im),
-						  gdk_imlib_move_mask(im));
+		imageview->image = gtk_pixmap_new(pixmap, mask);
 
 		gtk_scrolled_window_add_with_viewport
 			(GTK_SCROLLED_WINDOW(imageview->scrolledwin),
 			 imageview->image);
 	} else
-		gtk_pixmap_set(GTK_PIXMAP(imageview->image),
-			       gdk_imlib_move_image(im),
-			       gdk_imlib_move_mask(im));      
+		gtk_pixmap_set(GTK_PIXMAP(imageview->image), pixmap, mask);
+
+	gdk_pixmap_unref(pixmap);
 
 	gtk_widget_show(imageview->image);
-
-	gdk_imlib_destroy_image(im);
-
-	if (imageview->messageview->mainwin)
-		main_window_cursor_normal(imageview->messageview->mainwin);
 }
 #else
 void imageview_show_image(ImageView *imageview, MimeInfo *mimeinfo,
@@ -212,10 +231,20 @@ void imageview_clear(ImageView *imageview)
 	vadj = gtk_scrolled_window_get_vadjustment
 		(GTK_SCROLLED_WINDOW(imageview->scrolledwin));
 	gtk_adjustment_set_value(vadj, 0.0);
+
+	if (imageview->image_data) {
+#if HAVE_GDK_PIXBUF
+		gdk_pixbuf_unref((GdkPixbuf *)imageview->image_data);
+#elif HAVE_GDK_IMLIB
+		gdk_imlib_destroy_image((GdkImlibImage *)imageview->image_data);
+#endif
+		imageview->image_data = NULL;
+	}
 }
 
 void imageview_destroy(ImageView *imageview)
 {
+	imageview_clear(imageview);
 	g_free(imageview);
 }
 
@@ -226,6 +255,12 @@ static void get_resized_size(gint w, gint h, gint aw, gint ah,
 	gfloat hratio = 1.0;
 	gfloat ratio  = 1.0;
 
+	if (w <= aw && h <= ah) {
+		*sw = w;
+		*sh = h;
+		return;
+	}
+
 	if (w > aw)
 		wratio = (gfloat)aw / (gfloat)w;
 	if (h > ah)
@@ -236,9 +271,38 @@ static void get_resized_size(gint w, gint h, gint aw, gint ah,
 	*sw = (gint)(w * ratio);
 	*sh = (gint)(h * ratio);
 
-	/* be paranoid */
-	if (*sw <= 0 || *sh <= 0) {
-		*sw = w;
-		*sh = h;
+	/* restrict minimum size */
+	if (*sw < 16 || *sh < 16) {
+		wratio = 16.0 / (gfloat)w;
+		hratio = 16.0 / (gfloat)h;
+		ratio = (wratio > hratio) ? wratio : hratio;
+		if (ratio >= 1.0) {
+			*sw = w;
+			*sh = h;
+		} else {
+			*sw = (gint)(w * ratio);
+			*sh = (gint)(h * ratio);
+		}
 	}
 }
+
+static gint button_press_cb(GtkWidget *widget, GdkEventButton *event,
+			    gpointer data)
+{
+	ImageView *imageview = (ImageView *)data;
+
+	if (event->button == 1 && imageview->image) {
+		imageview_show_image(imageview, NULL, NULL, !imageview->resize);
+		return TRUE;
+	}
+	return FALSE;
+}
+
+static void size_allocate_cb(GtkWidget *widget,GtkAllocation *allocation,
+			     gpointer data)
+{
+	ImageView *imageview = (ImageView *)data;
+
+	if (imageview->resize)
+		imageview_show_image(imageview, NULL, NULL, TRUE);
+}
