@@ -270,7 +270,9 @@ static gint imap_cmd_login	(IMAPSession	*session,
 				 const gchar	*pass);
 static gint imap_cmd_logout	(IMAPSession	*session);
 static gint imap_cmd_noop	(IMAPSession	*session);
+#if USE_SSL
 static gint imap_cmd_starttls	(IMAPSession	*session);
+#endif
 static gint imap_cmd_namespace	(IMAPSession	*session,
 				 gchar	       **ns_str);
 static gint imap_cmd_list	(IMAPSession	*session,
@@ -438,16 +440,14 @@ static IMAPSession *imap_session_get(Folder *folder)
 
 	if (!rfolder->session) {
 		rfolder->session = imap_session_new(folder->account);
-		if (rfolder->session) {
+		if (rfolder->session)
 			imap_parse_namespace(IMAP_SESSION(rfolder->session),
 					     IMAP_FOLDER(folder));
-			rfolder->session->last_access_time = time(NULL);
-		}
 		return IMAP_SESSION(rfolder->session);
 	}
 
-	if (time(NULL) - rfolder->session->last_access_time < SESSION_TIMEOUT_INTERVAL) {
-		rfolder->session->last_access_time = time(NULL);
+	if (time(NULL) - rfolder->session->last_access_time <
+		SESSION_TIMEOUT_INTERVAL) {
 		return IMAP_SESSION(rfolder->session);
 	}
 
@@ -462,8 +462,6 @@ static IMAPSession *imap_session_get(Folder *folder)
 					     IMAP_FOLDER(folder));
 	}
 
-	if (rfolder->session)
-		rfolder->session->last_access_time = time(NULL);
 	return IMAP_SESSION(rfolder->session);
 }
 
@@ -2156,6 +2154,8 @@ static GSList *imap_get_uncached_messages(IMAPSession *session,
 
 	g_string_free(str, TRUE);
 
+	session_set_access_time(SESSION(session));
+
 	return newlist;
 }
 
@@ -3052,11 +3052,13 @@ static gint imap_cmd_noop(IMAPSession *session)
 	return imap_cmd_ok(session, NULL);
 }
 
+#if USE_SSL
 static gint imap_cmd_starttls(IMAPSession *session)
 {
 	imap_cmd_gen_send(session, "STARTTLS");
 	return imap_cmd_ok(session, NULL);
 }
+#endif
 
 #define THROW(err) { ok = err; goto catch; }
 
@@ -3112,6 +3114,7 @@ static gint imap_cmd_do_select(IMAPSession *session, const gchar *folder,
 	GPtrArray *argbuf;
 	gchar *select_cmd;
 	gchar *folder_;
+	guint uid_validity_;
 
 	*exists = *recent = *unseen = *uid_validity = 0;
 	argbuf = g_ptr_array_new();
@@ -3144,11 +3147,12 @@ static gint imap_cmd_do_select(IMAPSession *session, const gchar *folder,
 
 	resp_str = search_array_contain_str(argbuf, "UIDVALIDITY");
 	if (resp_str) {
-		if (sscanf(resp_str, "OK [UIDVALIDITY %u] ", uid_validity)
+		if (sscanf(resp_str, "OK [UIDVALIDITY %u] ", &uid_validity_)
 		    != 1) {
 			g_warning("imap_cmd_select(): invalid UIDVALIDITY line.\n");
 			THROW;
 		}
+		*uid_validity = uid_validity_;
 	}
 
 	resp_str = search_array_contain_str(argbuf, "UNSEEN");
@@ -3330,7 +3334,7 @@ static gint imap_cmd_append(IMAPSession *session, const gchar *destfolder,
 	gint size;
 	gchar *destfolder_;
 	gchar *flag_str;
-	guint32 new_uid_;
+	guint new_uid_;
 	gchar *ret = NULL;
 	gchar buf[BUFFSIZE];
 	FILE *fp;
@@ -3544,6 +3548,8 @@ static gint imap_cmd_gen_recv(IMAPSession *session, gchar **ret)
 
 	log_print("IMAP4< %s\n", *ret);
 
+	session_set_access_time(SESSION(session));
+
 	return IMAP_SUCCESS;
 }
 
