@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2002 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2003 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,9 +27,6 @@
 #include <glib.h>
 #include <time.h>
 
-#if USE_SSL
-#  include "ssl.h"
-#endif
 #include "folder.h"
 #include "session.h"
 #include "procmsg.h"
@@ -38,11 +35,17 @@ typedef struct _IMAPFolder	IMAPFolder;
 typedef struct _IMAPSession	IMAPSession;
 typedef struct _IMAPNameSpace	IMAPNameSpace;
 
-#include "prefs_account.h"
-
 #define IMAP_FOLDER(obj)	((IMAPFolder *)obj)
 #define IMAP_SESSION(obj)	((IMAPSession *)obj)
 
+#include "prefs_account.h"
+
+typedef enum
+{
+	IMAP_AUTH_LOGIN		= 1 << 0,
+	IMAP_AUTH_CRAM_MD5	= 1 << 1
+} IMAPAuthType;
+
 struct _IMAPFolder
 {
 	RemoteFolder rfolder;
@@ -57,8 +60,13 @@ struct _IMAPSession
 {
 	Session session;
 
+	gboolean authenticated;
+
+	gchar **capability;
+	gboolean uidplus;
+
 	gchar *mbox;
-	time_t last_access_time;
+	guint cmd_count;
 };
 
 struct _IMAPNameSpace
@@ -92,76 +100,15 @@ typedef enum
 #define IMAP_IS_DELETED(flags)	((flags & IMAP_FLAG_DELETED) != 0)
 #define IMAP_IS_DRAFT(flags)	((flags & IMAP_FLAG_DRAFT) != 0)
 
-Folder	*imap_folder_new		(const gchar	*name,
-					 const gchar	*path);
-void	 imap_folder_destroy		(Folder		*folder);
-
-#if USE_SSL
-Session *imap_session_new		(const gchar	*server,
-					 gushort	 port,
-					 const gchar	*user,
-					 const gchar	*pass,
-					 SSLType	 ssl_type);
-#else
-Session *imap_session_new		(const gchar	*server,
-					 gushort	 port,
-					 const gchar	*user,
-					 const gchar	*pass);
-#endif
-void imap_session_destroy		(Session	*session);
-void imap_session_destroy_all		(void);
-
-GSList *imap_get_msg_list		(Folder		*folder,
-					 FolderItem	*item,
-					 gboolean	 use_cache);
-gchar *imap_fetch_msg			(Folder		*folder,
-					 FolderItem	*item,
-					 gint		 uid);
-gint imap_add_msg			(Folder		*folder,
-					 FolderItem	*dest,
-					 const gchar	*file,
-					 gboolean	 remove_source);
-
-gint imap_move_msg			(Folder		*folder,
-					 FolderItem	*dest,
-					 MsgInfo	*msginfo);
-gint imap_move_msgs_with_dest		(Folder		*folder,
-					 FolderItem	*dest,
-					 GSList		*msglist);
-gint imap_copy_msg			(Folder		*folder,
-					 FolderItem	*dest,
-					 MsgInfo	*msginfo);
-gint imap_copy_msgs_with_dest		(Folder		*folder,
-					 FolderItem	*dest,
-					 GSList		*msglist);
-
-gint imap_remove_msg			(Folder		*folder,
-					 FolderItem	*item,
-					 gint		 uid);
-gint imap_remove_all_msg		(Folder		*folder,
-					 FolderItem	*item);
-gboolean imap_is_msg_changed		(Folder		*folder,
-					 FolderItem	*item,
-					 MsgInfo	*msginfo);
-
-gint imap_scan_folder			(Folder		*folder,
-					 FolderItem	*item);
-void imap_scan_tree			(Folder		*folder);
-
-gint imap_create_tree			(Folder		*folder);
-
-FolderItem *imap_create_folder		(Folder		*folder,
-					 FolderItem	*parent,
-					 const gchar	*name);
-gint imap_rename_folder			(Folder		*folder,
-					 FolderItem	*item,
-					 const gchar	*name);
-gint imap_remove_folder			(Folder		*folder,
-					 FolderItem	*item);
+FolderClass *imap_get_class		(void);
 
 gint imap_msg_set_perm_flags		(MsgInfo	*msginfo,
 					 MsgPermFlags	 flags);
 gint imap_msg_unset_perm_flags		(MsgInfo	*msginfo,
 					 MsgPermFlags	 flags);
+gint imap_msg_list_set_perm_flags	(GSList		*msglist,
+					 MsgPermFlags	 flags);
+gint imap_msg_list_unset_perm_flags	(GSList		*msglist,
+					 MsgPermFlags	 flags);
 
 #endif /* __IMAP_H__ */
