@@ -168,7 +168,8 @@ static void inc_finished(MainWindow *mainwin, gboolean new_messages)
 		folderview_select(mainwin->folderview, item);
 	} else if (prefs_common.scan_all_after_inc) {
 		item = mainwin->summaryview->folder_item;
-		folderview_update_item(item, TRUE);
+		if (item)
+			folderview_update_item(item, TRUE);
 	}
 }
 
@@ -187,7 +188,7 @@ void inc_mail(MainWindow *mainwin)
 
 	if (prefs_common.use_extinc && prefs_common.extinc_cmd) {
 		/* external incorporating program */
-		if (execute_command_line(prefs_common.extinc_cmd, FALSE) < 0) {
+		if (execute_command_line(prefs_common.extinc_cmd, FALSE) != 0) {
 			main_window_unlock(mainwin);
 			inc_autocheck_timer_set();
 			return;
@@ -411,7 +412,7 @@ static IncSession *inc_session_new(PrefsAccount *account)
 
 	g_return_val_if_fail(account != NULL, NULL);
 
-	if (account->protocol != A_POP3 && account->protocol != A_APOP)
+	if (account->protocol != A_POP3)
 		return NULL;
 	if (!account->recv_server || !account->userid)
 		return NULL;
@@ -1002,11 +1003,12 @@ static gint inc_recv_message(Session *session, const gchar *msg, gpointer data)
 static gint inc_drop_message(Pop3Session *session, const gchar *file)
 {
 	FolderItem *inbox;
-	FolderItem *dropfolder;
+	GSList *cur;
+	FilterInfo *fltinfo;
 	IncSession *inc_session = (IncSession *)(SESSION(session)->data);
 	gint val;
 
-	g_return_val_if_fail(inc_session != NULL, -1);
+	g_return_val_if_fail(inc_session != NULL, DROP_ERROR);
 
 	if (session->ac_prefs->inbox) {
 		inbox = folder_find_item_from_identifier
@@ -1015,41 +1017,52 @@ static gint inc_drop_message(Pop3Session *session, const gchar *file)
 			inbox = folder_get_default_inbox();
 	} else
 		inbox = folder_get_default_inbox();
-	if (!inbox) {
-		unlink(file);
-		return -1;
+	if (!inbox)
+		return DROP_ERROR;
+
+	fltinfo = filter_info_new();
+	fltinfo->account = session->ac_prefs;
+	fltinfo->flags.perm_flags = MSG_NEW|MSG_UNREAD;
+	fltinfo->flags.tmp_flags = MSG_RECEIVED;
+
+	if (session->ac_prefs->filter_on_recv)
+		filter_apply(prefs_common.fltlist, file, fltinfo);
+
+	if (!fltinfo->drop_done) {
+		if (folder_item_add_msg
+			(inbox, file, &fltinfo->flags, FALSE) < 0) {
+			filter_info_free(fltinfo);
+			return DROP_ERROR;
+		}
+		fltinfo->dest_list = g_slist_append(fltinfo->dest_list, inbox);
 	}
 
-	if (session->ac_prefs->filter_on_recv) {
-		dropfolder =
-			filter_get_dest_folder(prefs_common.fltlist, file);
-		if (!dropfolder) dropfolder = inbox;
-		else if (!strcmp(dropfolder->path, FILTER_NOT_RECEIVE)) {
-			unlink(file);
-			return 1;
-		}
-	} else
-		dropfolder = inbox;
+	for (cur = fltinfo->dest_list; cur != NULL; cur = cur->next) {
+		FolderItem *drop_folder = (FolderItem *)cur->data;
 
-	val = GPOINTER_TO_INT(g_hash_table_lookup
-			      (inc_session->folder_table, dropfolder));
-	if (val == 0) {
-		folder_item_scan(dropfolder);
-		/* force updating */
-		if (FOLDER_IS_LOCAL(dropfolder->folder))
-			dropfolder->mtime = 0;
-		g_hash_table_insert(inc_session->folder_table, dropfolder,
+		val = GPOINTER_TO_INT(g_hash_table_lookup
+				      (inc_session->folder_table, drop_folder));
+		if (val == 0) {
+			/* force updating */
+			if (FOLDER_IS_LOCAL(drop_folder->folder))
+				drop_folder->mtime = 0;
+			g_hash_table_insert(inc_session->folder_table, drop_folder,
+					    GINT_TO_POINTER(1));
+		}
+		g_hash_table_insert(inc_session->tmp_folder_table, drop_folder,
 				    GINT_TO_POINTER(1));
 	}
-	g_hash_table_insert(inc_session->tmp_folder_table, dropfolder,
-			    GINT_TO_POINTER(1));
 
-	if (folder_item_add_msg(dropfolder, file, NULL, TRUE) < 0) {
-		unlink(file);
-		return -1;
-	}
+	if (fltinfo->actions[FLT_ACTION_NOT_RECEIVE] == TRUE)
+		val = DROP_DONT_RECEIVE;
+	else if (fltinfo->actions[FLT_ACTION_DELETE] == TRUE)
+		val = DROP_DELETE;
+	else
+		val = DROP_OK;
 
-	return 0;
+	filter_info_free(fltinfo);
+
+	return val;
 }
 
 static void inc_put_error(IncState istate, const gchar *msg)
@@ -1189,13 +1202,22 @@ static gint inc_dialog_delete_cb(GtkWidget *widget, GdkEventAny *event,
 
 static gint inc_spool(void)
 {
-	gchar *mbox, *logname;
+	gchar *spool_path;
+	gchar *mbox;
 	gint msgs;
 
-	logname = g_get_user_name();
-	mbox = g_strconcat(prefs_common.spool_path
-			   ? prefs_common.spool_path : DEFAULT_SPOOL_PATH,
-			   G_DIR_SEPARATOR_S, logname, NULL);
+	spool_path = prefs_common.spool_path
+		? prefs_common.spool_path : DEFAULT_SPOOL_PATH;
+	if (is_file_exist(spool_path))
+		mbox = g_strdup(spool_path);
+	else if (is_dir_exist(spool_path))
+		mbox = g_strconcat(spool_path, G_DIR_SEPARATOR_S,
+				   g_get_user_name(), NULL);
+	else {
+		debug_print("%s: local mailbox not found.\n", spool_path);
+		return -1;
+	}
+
 	msgs = get_spool(folder_get_default_inbox(), mbox);
 	g_free(mbox);
 
@@ -1213,7 +1235,7 @@ static gint get_spool(FolderItem *dest, const gchar *mbox)
 	g_return_val_if_fail(mbox != NULL, -1);
 
 	if (!is_file_exist(mbox) || (size = get_file_size(mbox)) == 0) {
-		debug_print(_("no messages in local mailbox.\n"));
+		debug_print("%s: no messages in local mailbox.\n", mbox);
 		return 0;
 	} else if (size < 0)
 		return -1;
