@@ -39,6 +39,7 @@
 #include "intl.h"
 #include "inputdialog.h"
 #include "manage_window.h"
+#include "inc.h"
 #include "gtkutils.h"
 #include "utils.h"
 
@@ -233,6 +234,8 @@ static gchar *input_dialog_open(const gchar *title, const gchar *message,
 
 	ack = fin = FALSE;
 
+	inc_lock();
+
 	while (fin == FALSE)
 		gtk_main_iteration();
 
@@ -257,6 +260,8 @@ static gchar *input_dialog_open(const gchar *title, const gchar *message,
 
 	GTK_EVENTS_FLUSH();
 
+	inc_unlock();
+
 	debug_print("return string = %s\n", str ? str : "(none)");
 	return str;
 }
