@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2003 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2004 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -55,12 +55,12 @@
 #include "folderview.h"
 #include "summaryview.h"
 #include "prefs_common.h"
-#include "prefs_filter.h"
 #include "prefs_account.h"
 #include "prefs_actions.h"
 #include "prefs_display_header.h"
 #include "account.h"
 #include "procmsg.h"
+#include "filter.h"
 #include "inc.h"
 #include "import.h"
 #include "manage_window.h"
@@ -105,9 +105,11 @@ static struct RemoteCmd {
 
 static void parse_cmd_opt(int argc, char *argv[]);
 
+#if 0
 #if USE_GPGME
 static void idle_function_for_gpgme(void);
 #endif /* USE_GPGME */
+#endif /* 0 */
 
 static gint prohibit_duplicate_launch	(void);
 static gint lock_socket_remove		(void);
@@ -205,8 +207,6 @@ int main(int argc, char *argv[])
 	gtk_rc_parse(userrc);
 	g_free(userrc);
 
-	gtk_rc_parse("./gtkrc");
-
 	userrc = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, MENU_RC, NULL);
 	gtk_item_factory_parse_rc(userrc);
 	g_free(userrc);
@@ -235,23 +235,40 @@ int main(int argc, char *argv[])
 	prefs_common_read_config();
 
 #if USE_GPGME
-	if (gpgme_check_engine()) {  /* Also does some gpgme init */
+	if (gpgme_check_version("0.4.5")) {  /* Also does some gpgme init */
+        gpgme_engine_info_t engineInfo;
 		rfc2015_disable_all();
-		debug_print("gpgme_engine_version:\n%s\n",
-			    gpgme_get_engine_info());
 
+        gpgme_set_locale(NULL, LC_CTYPE, setlocale(LC_CTYPE, NULL));
+        gpgme_set_locale(NULL, LC_MESSAGES, setlocale(LC_MESSAGES, NULL));
+
+        if (!gpgme_get_engine_info(&engineInfo)) {
+            while (engineInfo) {
+                debug_print("GpgME Protocol: %s\n      Version: %s\n",
+                    gpgme_get_protocol_name(engineInfo->protocol),
+                    engineInfo->version);
+                engineInfo = engineInfo->next;
+            }
+        }
+    } else {
 		if (prefs_common.gpg_warning) {
 			AlertValue val;
 
 			val = alertpanel_message_with_disable
 				(_("Warning"),
 				 _("GnuPG is not installed properly, or its version is too old.\n"
-				   "OpenPGP support disabled."));
+				   "OpenPGP support disabled."),
+				 ALERT_WARNING);
 			if (val & G_ALERTDISABLE)
 				prefs_common.gpg_warning = FALSE;
 		}
 	}
-	gpgme_register_idle(idle_function_for_gpgme);
+    /* FIXME: This function went away.  We can either block until gpgme
+     * operations finish (currently implemented) or register callbacks
+     * with the gtk main loop via the gpgme io callback interface instead.
+     *
+	 * gpgme_register_idle(idle_function_for_gpgme);
+     */
 #endif
 
 	sock_set_io_timeout(prefs_common.io_timeout_secs);
@@ -467,9 +484,9 @@ void app_will_exit(GtkWidget *widget, gpointer data)
 
 	main_window_get_size(mainwin);
 	main_window_get_position(mainwin);
-	prefs_common_save_config();
+	prefs_common_write_config();
 	prefs_filter_write_config();
-	account_save_config_all();
+	account_write_config_all();
 	addressbook_export_to_file();
 
 	filename = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, MENU_RC, NULL);
@@ -489,6 +506,7 @@ void app_will_exit(GtkWidget *widget, gpointer data)
 	gtk_main_quit();
 }
 
+#if 0
 #if USE_GPGME
 static void idle_function_for_gpgme(void)
 {
@@ -496,6 +514,7 @@ static void idle_function_for_gpgme(void)
 		gtk_main_iteration();
 }
 #endif /* USE_GPGME */
+#endif /* 0 */
 
 static gchar *get_socket_name(void)
 {
@@ -700,6 +719,9 @@ static void send_queue(void)
 {
 	GList *list;
 
+	if (!main_window_toggle_online_if_offline(main_window_get()))
+		return;
+
 	for (list = folder_get_list(); list != NULL; list = list->next) {
 		Folder *folder = list->data;
 
