@@ -193,9 +193,17 @@ static void delete_folder_cb	 (MainWindow	*mainwin,
 static void update_folderview_cb (MainWindow	*mainwin,
 				  guint		 action,
 				  GtkWidget	*widget);
+
 static void add_mailbox_cb	 (MainWindow	*mainwin,
 				  guint		 action,
 				  GtkWidget	*widget);
+static void remove_mailbox_cb	 (MainWindow	*mainwin,
+				  guint		 action,
+				  GtkWidget	*widget);
+static void rebuild_tree_cb	 (MainWindow	*mainwin,
+				  guint		 action,
+				  GtkWidget	*widget);
+
 static void import_mbox_cb	 (MainWindow	*mainwin,
 				  guint		 action,
 				  GtkWidget	*widget);
@@ -448,13 +456,22 @@ static GtkItemFactoryEntry mainwin_entries[] =
 						NULL, new_folder_cb, 0, NULL},
 	{N_("/_File/_Folder/_Rename folder..."),NULL, rename_folder_cb, 0, NULL},
 	{N_("/_File/_Folder/_Delete folder"),	NULL, delete_folder_cb, 0, NULL},
-	{N_("/_File/_Folder/---"),			NULL, NULL, 0, "<Separator>"},
-	{N_("/_File/_Folder/_Check for new messages in all folders"),
+	{N_("/_File/_Mailbox"),			NULL, NULL, 0, "<Branch>"},
+	{N_("/_File/_Mailbox/Add _mailbox..."),	NULL, add_mailbox_cb, 0, NULL},
+	{N_("/_File/_Mailbox/_Remove mailbox"),	NULL, remove_mailbox_cb, 0, NULL},
+	{N_("/_File/_Mailbox/---"),		NULL, NULL, 0, "<Separator>"},
+	{N_("/_File/_Mailbox/_Check for new messages"),
 						NULL, update_folderview_cb, 0, NULL},
-	{N_("/_File/_Add mailbox..."),		NULL, add_mailbox_cb, 0, NULL},
+	{N_("/_File/_Mailbox/Check for new messages in _all mailboxes"),
+						NULL, update_folderview_cb, 1, NULL},
+	{N_("/_File/_Mailbox/---"),		NULL, NULL, 0, "<Separator>"},
+	{N_("/_File/_Mailbox/R_ebuild folder tree"),
+						NULL, rebuild_tree_cb, 0, NULL},
+	{N_("/_File/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_File/_Import mbox file..."),	NULL, import_mbox_cb, 0, NULL},
 	{N_("/_File/_Export to mbox file..."),	NULL, export_mbox_cb, 0, NULL},
-	{N_("/_File/Empty _trash"),		NULL, empty_trash_cb, 0, NULL},
+	{N_("/_File/---"),			NULL, NULL, 0, "<Separator>"},
+	{N_("/_File/Empty all _trash"),		NULL, empty_trash_cb, 0, NULL},
 	{N_("/_File/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_File/_Save as..."),		"<control>S", save_as_cb, 0, NULL},
 	{N_("/_File/_Print..."),		NULL, print_cb, 0, NULL},
@@ -1469,14 +1486,14 @@ void main_window_empty_trash(MainWindow *mainwin, gboolean confirm)
 	GList *list;
 
 	if (confirm) {
-		if (alertpanel(_("Empty trash"),
-			       _("Empty all messages in trash?"),
+		if (alertpanel(_("Empty all trash"),
+			       _("Empty messages in all trash?"),
 			       _("Yes"), _("No"), NULL) != G_ALERTDEFAULT)
 			return;
 		manage_window_focus_in(mainwin->window, NULL, NULL);
 	}
 
-	procmsg_empty_trash();
+	procmsg_empty_all_trash();
 	statusbar_pop_all();
 
 	for (list = folder_get_list(); list != NULL; list = list->next) {
@@ -1541,7 +1558,16 @@ typedef enum
 	M_THREADED	      = 1 << 7,
 	M_UNTHREADED	      = 1 << 8,
 	M_ALLOW_DELETE	      = 1 << 9,
-	M_INC_ACTIVE	      = 1 << 10
+	M_INC_ACTIVE	      = 1 << 10,
+
+	M_FOLDER_NEWOK	      = 1 << 11,
+	M_FOLDER_RENOK	      = 1 << 12,
+	M_FOLDER_DELOK	      = 1 << 13,
+	M_MBOX_ADDOK	      = 1 << 14,
+	M_MBOX_RMOK	      = 1 << 15,
+	M_MBOX_CHKOK	      = 1 << 16,
+	M_MBOX_CHKALLOK	      = 1 << 17,
+	M_MBOX_REBUILDOK      = 1 << 18
 } SensitiveCond;
 
 static SensitiveCond main_window_get_current_state(MainWindow *mainwin)
@@ -1581,6 +1607,29 @@ static SensitiveCond main_window_get_current_state(MainWindow *mainwin)
 	if (inc_is_active())
 		state |= M_INC_ACTIVE;
 
+	item = folderview_get_selected_item(mainwin->folderview);
+	if (item) {
+		state |= M_FOLDER_NEWOK;
+		if (item->parent == NULL) {
+			state |= M_MBOX_RMOK;
+			state |= M_MBOX_CHKOK;
+		}
+		if (FOLDER_IS_LOCAL(item->folder) ||
+		    FOLDER_TYPE(item->folder) == F_IMAP) {
+			if (item->parent == NULL)
+				state |= M_MBOX_REBUILDOK;
+			else if (item->stype == F_NORMAL) {
+				state |= M_FOLDER_RENOK;
+				state |= M_FOLDER_DELOK;
+			}
+		} else if (FOLDER_TYPE(item->folder) == F_NEWS) {
+			if (item->parent != NULL)
+				state |= M_FOLDER_DELOK;
+		}
+	}
+	state |= M_MBOX_ADDOK;
+	state |= M_MBOX_CHKALLOK;
+
 	return state;
 }
 
@@ -1650,11 +1699,19 @@ void main_window_set_menu_sensitive(MainWindow *mainwin)
 		gchar *const entry;
 		SensitiveCond cond;
 	} entry[] = {
-		{"/File/Folder"                , M_UNLOCKED},
-		{"/File/Add mailbox..."        , M_UNLOCKED},
-		{"/File/Import mbox file..."   , M_UNLOCKED},
-		{"/File/Export to mbox file...", M_UNLOCKED},
-		{"/File/Empty trash"           , M_UNLOCKED},
+		{"/File/Folder/Create new folder...", M_UNLOCKED|M_FOLDER_NEWOK},
+		{"/File/Folder/Rename folder..."    , M_UNLOCKED|M_FOLDER_RENOK},
+		{"/File/Folder/Delete folder"       , M_UNLOCKED|M_FOLDER_DELOK},
+		{"/File/Mailbox/Add mailbox..."     , M_UNLOCKED|M_MBOX_ADDOK},
+		{"/File/Mailbox/Remove mailbox"     , M_UNLOCKED|M_MBOX_RMOK},
+		{"/File/Mailbox/Check for new messages"
+						    , M_UNLOCKED|M_MBOX_CHKOK},
+		{"/File/Mailbox/Check for new messages in all mailboxes"
+						    , M_UNLOCKED|M_MBOX_CHKALLOK},
+		{"/File/Mailbox/Rebuild folder tree", M_UNLOCKED|M_MBOX_REBUILDOK},
+		{"/File/Import mbox file..."        , M_UNLOCKED},
+		{"/File/Export to mbox file..."     , M_UNLOCKED},
+		{"/File/Empty all trash"            , M_UNLOCKED},
 
 		{"/File/Save as..."  , M_SINGLE_TARGET_EXIST},
 		{"/File/Print..."    , M_TARGET_EXIST},
@@ -2473,19 +2530,6 @@ static void message_window_size_allocate_cb(GtkWidget *widget,
 	main_window_get_size(mainwin);
 }
 
-static void add_mailbox_cb(MainWindow *mainwin, guint action,
-			   GtkWidget *widget)
-{
-	main_window_add_mailbox(mainwin);
-}
-
-static void update_folderview_cb(MainWindow *mainwin, guint action,
-				 GtkWidget *widget)
-{
-	summary_show(mainwin->summaryview, NULL, FALSE);
-	folderview_check_new_all();
-}
-
 static void new_folder_cb(MainWindow *mainwin, guint action,
 			  GtkWidget *widget)
 {
@@ -2504,6 +2548,33 @@ static void delete_folder_cb(MainWindow *mainwin, guint action,
 	folderview_delete_folder(mainwin->folderview);
 }
 
+static void add_mailbox_cb(MainWindow *mainwin, guint action,
+			   GtkWidget *widget)
+{
+	main_window_add_mailbox(mainwin);
+}
+
+static void remove_mailbox_cb(MainWindow *mainwin, guint action,
+			      GtkWidget *widget)
+{
+	folderview_remove_mailbox(mainwin->folderview);
+}
+
+static void update_folderview_cb(MainWindow *mainwin, guint action,
+				 GtkWidget *widget)
+{
+	if (action == 0)
+		folderview_check_new_selected(mainwin->folderview);
+	else
+		folderview_check_new_all();
+}
+
+static void rebuild_tree_cb(MainWindow *mainwin, guint action,
+			    GtkWidget *widget)
+{
+	folderview_rebuild_tree(mainwin->folderview);
+}
+
 static void import_mbox_cb(MainWindow *mainwin, guint action,
 			   GtkWidget *widget)
 {
@@ -2693,6 +2764,9 @@ static void send_queue_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
 {
 	GList *list;
 
+	if (!main_window_toggle_online_if_offline(mainwin))
+		return;
+
 	for (list = folder_get_list(); list != NULL; list = list->next) {
 		Folder *folder = list->data;
 
@@ -3027,7 +3101,7 @@ static void prefs_common_open_cb(MainWindow *mainwin, guint action,
 static void prefs_filter_open_cb(MainWindow *mainwin, guint action,
 				 GtkWidget *widget)
 {
-	prefs_filter_open(NULL, NULL, NULL);
+	prefs_filter_open(NULL, NULL);
 }
 
 static void prefs_template_open_cb(MainWindow *mainwin, guint action,
