@@ -99,17 +99,17 @@ gint proc_mbox(FolderItem *dest, const gchar *mbox, GHashTable *folder_table)
 
 	do {
 		FILE *tmp_fp;
-		FolderItem *dropfolder;
+		GSList *cur;
 		gchar *startp, *endp, *rpath;
 		gint empty_line;
-		gint val;
 		gboolean is_next_msg = FALSE;
+		FilterInfo *fltinfo;
 
 		if ((tmp_fp = fopen(tmp_file, "wb")) == NULL) {
 			FILE_OP_ERROR(tmp_file, "fopen");
 			g_warning(_("can't open temporary file\n"));
-			fclose(mbox_fp);
 			g_free(tmp_file);
+			fclose(mbox_fp);
 			return -1;
 		}
 		if (change_file_mode_rw(tmp_fp, tmp_file) < 0)
@@ -194,42 +194,57 @@ gint proc_mbox(FolderItem *dest, const gchar *mbox, GHashTable *folder_table)
 		if (fclose(tmp_fp) == EOF) {
 			FILE_OP_ERROR(tmp_file, "fclose");
 			g_warning(_("can't write to temporary file\n"));
-			fclose(mbox_fp);
 			unlink(tmp_file);
 			g_free(tmp_file);
+			fclose(mbox_fp);
 			return -1;
 		}
 
-		if (folder_table) {
-			dropfolder = filter_get_dest_folder
-				(prefs_common.fltlist, tmp_file);
-			if (!dropfolder ||
-			    !strcmp(dropfolder->path, FILTER_NOT_RECEIVE))
-				dropfolder = dest;
-			val = GPOINTER_TO_INT(g_hash_table_lookup
-					      (folder_table, dropfolder));
-			if (val == 0) {
-				folder_item_scan(dropfolder);
-				/* force updating */
-				if (FOLDER_IS_LOCAL(dropfolder->folder))
-					dropfolder->mtime = 0;
-				g_hash_table_insert(folder_table, dropfolder,
-						    GINT_TO_POINTER(1));
+		fltinfo = filter_info_new();
+		fltinfo->flags.perm_flags = MSG_NEW|MSG_UNREAD;
+		fltinfo->flags.tmp_flags = MSG_RECEIVED;
+
+		if (folder_table)
+			filter_apply(prefs_common.fltlist, tmp_file, fltinfo);
+
+		if (fltinfo->actions[FLT_ACTION_MOVE] == FALSE &&
+		    fltinfo->actions[FLT_ACTION_DELETE] == FALSE) {
+			if (folder_item_add_msg(dest, tmp_file, &fltinfo->flags,
+						FALSE) < 0) {
+				filter_info_free(fltinfo);
+				unlink(tmp_file);
+				g_free(tmp_file);
+				fclose(mbox_fp);
+				return -1;
 			}
-		} else {
-			dropfolder = dest;
-			/* force updating */
-			if (FOLDER_IS_LOCAL(dropfolder->folder))
-				dropfolder->mtime = 0;
+			fltinfo->dest_list = g_slist_append(fltinfo->dest_list,
+							    dest);
 		}
 
-		if (folder_item_add_msg(dropfolder, tmp_file, NULL, TRUE) < 0) {
-			fclose(mbox_fp);
-			unlink(tmp_file);
-			g_free(tmp_file);
-			return -1;
+		for (cur = fltinfo->dest_list; cur != NULL; cur = cur->next) {
+			FolderItem *drop_folder = (FolderItem *)cur->data;
+			gint val = 0;
+
+			if (folder_table) {
+				val = GPOINTER_TO_INT(g_hash_table_lookup
+						      (folder_table,
+						       drop_folder));
+			}
+			if (val == 0) {
+				/* force updating */
+				if (FOLDER_IS_LOCAL(drop_folder->folder))
+					drop_folder->mtime = 0;
+				if (folder_table) {
+					g_hash_table_insert(folder_table,
+							    drop_folder,
+							    GINT_TO_POINTER(1));
+				}
+			}
 		}
 
+		filter_info_free(fltinfo);
+		unlink(tmp_file);
+
 		msgs++;
 	} while (from_line[0] != '\0');
 
