@@ -41,4 +41,7 @@ gint copy_mbox		(const gchar	*src,
 			 const gchar	*dest);
 void empty_mbox		(const gchar	*mbox);
 
+gint export_to_mbox	(FolderItem	*src,
+			 const gchar	*mbox);
+
 #endif /* __MBOX_H__ */
