@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2003 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2004 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 #include <gtk/gtkwidget.h>
 #include <gtk/gtkmenu.h>
 #include <gtk/gtkmenubar.h>
+#include <gtk/gtkcheckmenuitem.h>
 #include <gtk/gtkitemfactory.h>
 #include <gtk/gtkbutton.h>
 
@@ -169,6 +170,17 @@ void menu_set_sensitive_all(GtkMenuShell *menu_shell, gboolean sensitive)
 		gtk_widget_set_sensitive(GTK_WIDGET(cur->data), sensitive);
 }
 
+void menu_set_active(GtkItemFactory *ifactory, const gchar *path,
+		     gboolean is_active)
+{
+	GtkWidget *widget;
+
+	g_return_if_fail(ifactory != NULL);
+
+	widget = gtk_item_factory_get_item(ifactory, path);
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(widget), is_active);
+}
+
 void menu_button_position(GtkMenu *menu, gint *x, gint *y, gpointer user_data)
 {
 	GtkWidget *button;
@@ -231,3 +243,14 @@ gint menu_find_option_menu_index(GtkOptionMenu *optmenu, gpointer data,
 
 	return -1;
 }
+
+gpointer menu_get_option_menu_active_user_data(GtkOptionMenu *optmenu)
+{
+	GtkWidget *menu;
+	GtkWidget *menuitem;
+
+	menu = gtk_option_menu_get_menu(optmenu);
+	menuitem = gtk_menu_get_active(GTK_MENU(menu));
+
+	return gtk_object_get_user_data(GTK_OBJECT(menuitem));
+}
