@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2003 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2004 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -183,6 +183,8 @@ static GtkItemFactoryEntry msgview_entries[] =
 	 CODESET_ACTION(C_ISO_8859_5)},
 	{N_("/_View/_Code set/Cyrillic (KOI8-_R)"),
 	 CODESET_ACTION(C_KOI8_R)},
+	{N_("/_View/_Code set/Cyrillic (KOI8-U)"),
+	 CODESET_ACTION(C_KOI8_U)},
 	{N_("/_View/_Code set/Cyrillic (Windows-1251)"),
 	 CODESET_ACTION(C_CP1251)},
 	CODESET_SEPARATOR,
@@ -274,7 +276,6 @@ MessageView *messageview_create(void)
 	GtkWidget *vbox;
 	HeaderView *headerview;
 	TextView *textview;
-	ImageView *imageview;
 	MimeView *mimeview;
 
 	debug_print(_("Creating message view...\n"));
@@ -287,13 +288,11 @@ MessageView *messageview_create(void)
 	textview = textview_create();
 	textview->messageview = messageview;
 
-	imageview = imageview_create();
-	imageview->messageview = messageview;
-
 	mimeview = mimeview_create();
 	mimeview->textview = textview_create();
 	mimeview->textview->messageview = messageview;
-	mimeview->imageview = imageview;
+	mimeview->imageview = imageview_create();
+	mimeview->imageview->messageview = messageview;
 	mimeview->messageview = messageview;
 
 	vbox = gtk_vbox_new(FALSE, 0);
@@ -305,17 +304,20 @@ MessageView *messageview_create(void)
 
 	/* to remove without destroyed */
 	gtk_widget_ref(GTK_WIDGET_PTR(textview));
-	gtk_widget_ref(GTK_WIDGET_PTR(imageview));
 	gtk_widget_ref(GTK_WIDGET_PTR(mimeview));
 	gtk_widget_ref(GTK_WIDGET_PTR(mimeview->textview));
+	gtk_widget_ref(GTK_WIDGET_PTR(mimeview->imageview));
 
-	messageview->vbox       = vbox;
-	messageview->new_window = FALSE;
-	messageview->window     = NULL;
-	messageview->headerview = headerview;
-	messageview->textview   = textview;
-	messageview->imageview  = imageview;
-	messageview->mimeview   = mimeview;
+	messageview->vbox        = vbox;
+	messageview->new_window  = FALSE;
+	messageview->window      = NULL;
+	messageview->window_vbox = NULL;
+	messageview->headerview  = headerview;
+	messageview->textview    = textview;
+	messageview->mimeview    = mimeview;
+
+	messageview->statusbar     = NULL;
+	messageview->statusbar_cid = 0;
 
 	return messageview;
 }
@@ -324,8 +326,11 @@ MessageView *messageview_create_with_new_window(void)
 {
 	MessageView *msgview;
 	GtkWidget *window;
+	GtkWidget *window_vbox;
+	GtkWidget *body_vbox;
 	GtkWidget *menubar;
 	GtkItemFactory *ifactory;
+	GtkWidget *statusbar;
 	guint n_menu_entries;
 
 	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
@@ -337,6 +342,9 @@ MessageView *messageview_create_with_new_window(void)
 
 	msgview = messageview_create();
 
+	window_vbox = gtk_vbox_new(FALSE, 0);
+	gtk_container_add(GTK_CONTAINER(window), window_vbox);
+
 	gtk_signal_connect(GTK_OBJECT(window), "size_allocate",
 			   GTK_SIGNAL_FUNC(messageview_size_allocate_cb),
 			   msgview);
@@ -350,15 +358,28 @@ MessageView *messageview_create_with_new_window(void)
 	menubar = menubar_create(window, msgview_entries, n_menu_entries,
 				 "<MessageView>", msgview);
 	menu_factory_copy_rc("<Main>", "<MessageView>");
-	gtk_box_pack_start(GTK_BOX(msgview->vbox), menubar, FALSE, TRUE, 0);
-	gtk_box_reorder_child(GTK_BOX(msgview->vbox), menubar, 0);
+	gtk_box_pack_start(GTK_BOX(window_vbox), menubar, FALSE, TRUE, 0);
 
-	gtk_container_add(GTK_CONTAINER(window), GTK_WIDGET_PTR(msgview));
+	body_vbox = gtk_vbox_new(FALSE, BORDER_WIDTH);
+	gtk_container_set_border_width(GTK_CONTAINER(body_vbox), BORDER_WIDTH);
+	gtk_box_pack_start(GTK_BOX(window_vbox), body_vbox, TRUE, TRUE, 0);
+
+	gtk_box_pack_start(GTK_BOX(body_vbox), GTK_WIDGET_PTR(msgview),
+			   TRUE, TRUE, 0);
 	gtk_widget_grab_focus(msgview->textview->text);
+
+	statusbar = gtk_statusbar_new();
+	gtk_box_pack_end(GTK_BOX(body_vbox), statusbar, FALSE, FALSE, 0);
+	msgview->statusbar = statusbar;
+	msgview->statusbar_cid = gtk_statusbar_get_context_id
+		(GTK_STATUSBAR(statusbar), "Message View");
+
 	gtk_widget_show_all(window);
 
 	msgview->new_window = TRUE;
 	msgview->window = window;
+	msgview->window_vbox = window_vbox;
+	msgview->body_vbox = body_vbox;
 	msgview->visible = TRUE;
 
 	messageview_init(msgview);
@@ -375,7 +396,6 @@ void messageview_init(MessageView *messageview)
 {
 	headerview_init(messageview->headerview);
 	textview_init(messageview->textview);
-	imageview_init(messageview->imageview);
 	mimeview_init(messageview->mimeview);
 	/* messageview_set_font(messageview); */
 }
@@ -385,27 +405,35 @@ GList *messageview_get_window_list(void)
 	return messageview_list;
 }
 
-void messageview_show(MessageView *messageview, MsgInfo *msginfo,
+gint messageview_show(MessageView *messageview, MsgInfo *msginfo,
 		      gboolean all_headers)
 {
 	gchar *file;
 	MimeInfo *mimeinfo;
 
-	g_return_if_fail(msginfo != NULL);
+	g_return_val_if_fail(msginfo != NULL, -1);
 
 	mimeinfo = procmime_scan_message(msginfo);
-	g_return_if_fail(mimeinfo != NULL);
+	if (!mimeinfo) {
+		messageview_change_view_type(messageview, MVIEW_TEXT);
+		textview_show_error(messageview->textview);
+		return -1;
+	}
 
 	file = procmsg_get_message_file_path(msginfo);
 	if (!file) {
 		g_warning("can't get message file path.\n");
 		procmime_mimeinfo_free_all(mimeinfo);
-		return;
+		messageview_change_view_type(messageview, MVIEW_TEXT);
+		textview_show_error(messageview->textview);
+		return -1;
 	}
 
 	if (messageview->msginfo != msginfo) {
 		procmsg_msginfo_free(messageview->msginfo);
 		messageview->msginfo = procmsg_msginfo_get_full_info(msginfo);
+		if (!messageview->msginfo)
+			messageview->msginfo = procmsg_msginfo_copy(msginfo);
 	}
 	headerview_show(messageview->headerview, messageview->msginfo);
 
@@ -423,6 +451,8 @@ void messageview_show(MessageView *messageview, MsgInfo *msginfo,
 	}
 
 	g_free(file);
+
+	return 0;
 }
 
 static void messageview_change_view_type(MessageView *messageview,
@@ -445,6 +475,7 @@ static void messageview_change_view_type(MessageView *messageview,
 		gtkut_container_remove
 			(GTK_CONTAINER(GTK_WIDGET_PTR(messageview)),
 			 GTK_WIDGET_PTR(mimeview));
+		mimeview_clear(mimeview);
 
 		if (mimeview->vbox == GTK_WIDGET_PTR(textview)->parent)
 			gtkut_container_remove(GTK_CONTAINER(mimeview->vbox),
@@ -465,20 +496,19 @@ void messageview_clear(MessageView *messageview)
 	messageview_change_view_type(messageview, MVIEW_TEXT);
 	headerview_clear(messageview->headerview);
 	textview_clear(messageview->textview);
-	imageview_clear(messageview->imageview);
+	mimeview_clear(messageview->mimeview);
 }
 
 void messageview_destroy(MessageView *messageview)
 {
 	GtkWidget *textview  = GTK_WIDGET_PTR(messageview->textview);
-	GtkWidget *imageview = GTK_WIDGET_PTR(messageview->imageview);
+	GtkWidget *imageview = GTK_WIDGET_PTR(messageview->mimeview->imageview);
 	GtkWidget *mimeview  = GTK_WIDGET_PTR(messageview->mimeview);
 
 	messageview_list = g_list_remove(messageview_list, messageview);
 
 	headerview_destroy(messageview->headerview);
 	textview_destroy(messageview->textview);
-	imageview_destroy(messageview->imageview);
 	mimeview_destroy(messageview->mimeview);
 
 	procmsg_msginfo_free(messageview->msginfo);
@@ -826,7 +856,7 @@ static void create_filter_cb(gpointer data, guint action, GtkWidget *widget)
 
 	procmsg_get_filter_keyword(messageview->msginfo, &header, &key,
 				   (PrefsFilterType)action);
-	prefs_filter_open(messageview->msginfo, header, key);
+	prefs_filter_open(messageview->msginfo, header);
 
 	g_free(header);
 	g_free(key);
