@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2003 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2005 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -390,8 +390,16 @@ static gint mh_add_msgs(Folder *folder, FolderItem *dest, GSList *file_list,
 		dest->updated = TRUE;
 
 		if (fileinfo->flags) {
-			SET_DEST_MSG_FLAGS(fp, dest, dest->last_num,
-					   *fileinfo->flags);
+			if (MSG_IS_RECEIVED(*fileinfo->flags)) {
+				if (dest->unmarked_num == 0)
+					dest->new = 0;
+				dest->unmarked_num++;
+				procmsg_add_mark_queue(dest, dest->last_num,
+						       *fileinfo->flags);
+			} else {
+				SET_DEST_MSG_FLAGS(fp, dest, dest->last_num,
+						   *fileinfo->flags);
+			}
 			if (MSG_IS_NEW(*fileinfo->flags))
 				dest->new++;
 			if (MSG_IS_UNREAD(*fileinfo->flags))
@@ -399,10 +407,9 @@ static gint mh_add_msgs(Folder *folder, FolderItem *dest, GSList *file_list,
 		} else {
 			if (dest->unmarked_num == 0)
 				dest->new = 0;
-
+			dest->unmarked_num++;
 			dest->new++;
 			dest->unread++;
-			dest->unmarked_num++;
 		}
 	}
 
@@ -562,7 +569,7 @@ static gint mh_copy_msgs(Folder *folder, FolderItem *dest, GSList *msglist)
 
 	if (dest->opened)
 		fp = NULL;
-	if ((fp = procmsg_open_mark_file(dest, DATA_APPEND)) == NULL)
+	else if ((fp = procmsg_open_mark_file(dest, DATA_APPEND)) == NULL)
 		g_warning(_("Can't open mark file.\n"));
 
 	for (cur = msglist; cur != NULL; cur = cur->next) {
@@ -652,6 +659,7 @@ static gint mh_remove_all_msg(Folder *folder, FolderItem *item)
 	g_free(path);
 	if (val == 0) {
 		item->new = item->unread = item->total = 0;
+		item->last_num = 0;
 		item->updated = TRUE;
 	}
 
@@ -707,7 +715,7 @@ static gint mh_scan_folder_full(Folder *folder, FolderItem *item,
 		folder->ui_func(folder, item, folder->ui_func_data);
 
 	while ((d = readdir(dp)) != NULL) {
-		if ((num = to_number(d->d_name)) >= 0 &&
+		if ((num = to_number(d->d_name)) > 0 &&
 		    dirent_is_regular_file(d)) {
 			n_msg++;
 			if (max < num)
@@ -720,10 +728,10 @@ static gint mh_scan_folder_full(Folder *folder, FolderItem *item,
 	if (n_msg == 0)
 		item->new = item->unread = item->total = 0;
 	else if (count_sum) {
-		gint new, unread, total, min, max;
+		gint new, unread, total, min, max_;
 
 		procmsg_get_mark_sum
-			(item, &new, &unread, &total, &min, &max, 0);
+			(item, &new, &unread, &total, &min, &max_, 0);
 
 		if (n_msg > total) {
 			item->unmarked_num = new = n_msg - total;
@@ -958,7 +966,7 @@ static GSList *mh_get_uncached_msgs(GHashTable *msg_table, FolderItem *item)
 
 	if (msg_table) {
 		while ((d = readdir(dp)) != NULL) {
-			if ((num = to_number(d->d_name)) < 0) continue;
+			if ((num = to_number(d->d_name)) <= 0) continue;
 
 			msginfo = g_hash_table_lookup
 				(msg_table, GUINT_TO_POINTER(num));
@@ -981,7 +989,7 @@ static GSList *mh_get_uncached_msgs(GHashTable *msg_table, FolderItem *item)
 	} else {
 		/* discard all previous cache */
 		while ((d = readdir(dp)) != NULL) {
-			if (to_number(d->d_name) < 0) continue;
+			if (to_number(d->d_name) <= 0) continue;
 
 			msginfo = mh_parse_msg(d->d_name, item);
 			if (!msginfo) continue;
@@ -1016,7 +1024,6 @@ static GSList *mh_get_uncached_msgs(GHashTable *msg_table, FolderItem *item)
 
 static MsgInfo *mh_parse_msg(const gchar *file, FolderItem *item)
 {
-	struct stat s;
 	MsgInfo *msginfo;
 	MsgFlags flags;
 
@@ -1032,20 +1039,11 @@ static MsgInfo *mh_parse_msg(const gchar *file, FolderItem *item)
 		MSG_SET_TMP_FLAGS(flags, MSG_DRAFT);
 	}
 
-	if (stat(file, &s) < 0) {
-		FILE_OP_ERROR(file, "stat");
-		return NULL;
-	}
-	if (!S_ISREG(s.st_mode))
-		return NULL;
-
 	msginfo = procheader_parse_file(file, flags, FALSE);
 	if (!msginfo) return NULL;
 
 	msginfo->msgnum = atoi(file);
 	msginfo->folder = item;
-	msginfo->size = s.st_size;
-	msginfo->mtime = s.st_mtime;
 
 	return msginfo;
 }
@@ -1202,7 +1200,7 @@ static void mh_scan_tree_recursive(FolderItem *item)
 			}
 
 			mh_scan_tree_recursive(new_item);
-		} else if (to_number(d->d_name) != -1) n_msg++;
+		} else if (to_number(d->d_name) > 0) n_msg++;
 
 		g_free(entry);
 	}
