@@ -54,6 +54,7 @@ typedef enum {
 	POP3_RETR_RECV,
 	POP3_DELETE,
 	POP3_LOGOUT,
+	POP3_DONE,
 	POP3_ERROR,
 
 	N_POP3_STATE
@@ -85,9 +86,17 @@ typedef enum {
 typedef enum {
 	RECV_TIME_NONE     = 0,
 	RECV_TIME_RECEIVED = 1,
-	RECV_TIME_KEEP     = 2
+	RECV_TIME_KEEP     = 2,
+	RECV_TIME_DELETE   = 3
 } RecvTime;
 
+typedef enum {
+	DROP_OK = 0,
+	DROP_DONT_RECEIVE = 1,
+	DROP_DELETE = 2,
+	DROP_ERROR = -1
+} Pop3DropValue;
+
 struct _Pop3MsgInfo
 {
 	gint size;
@@ -135,8 +144,8 @@ struct _Pop3Session
 };
 
 #define POPBUFSIZE	512
-#define IDLEN		128
-
+/* #define IDLEN	128 */
+#define IDLEN		POPBUFSIZE
 
 Session *pop3_session_new	(PrefsAccount	*account);
 GHashTable *pop3_get_uidl_table	(PrefsAccount	*account);
