@@ -1,7 +1,7 @@
 /*
  * Sylpheed templates subsystem 
  * Copyright (C) 2001 Alexander Barinov
- * Copyright (C) 2001-2002 Hiroyuki Yamamoto
+ * Copyright (C) 2001-2004 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -45,8 +45,9 @@ static struct Templates {
 	GtkWidget *ok_btn;
 	GtkWidget *clist_tmpls;
 	GtkWidget *entry_name;
-	GtkWidget *entry_subject;
 	GtkWidget *entry_to;
+	GtkWidget *entry_cc;
+	GtkWidget *entry_subject;
 	GtkWidget *text_value;
 } templates;
 
@@ -111,6 +112,7 @@ static void prefs_template_window_create(void)
 	GtkWidget         *entry_name;
 	GtkWidget       *table;
 	GtkWidget         *entry_to;
+	GtkWidget         *entry_cc;
 	GtkWidget         *entry_subject;
 	GtkWidget       *scroll2;
 	GtkWidget         *text_value;
@@ -161,7 +163,7 @@ static void prefs_template_window_create(void)
 	gtk_box_pack_start(GTK_BOX(hbox1), entry_name, TRUE, TRUE, 0);
 
 	/* table for headers */
-	table = gtk_table_new(2, 2, FALSE);
+	table = gtk_table_new(3, 2, FALSE);
 	gtk_widget_show(table);
 	gtk_box_pack_start(GTK_BOX(vbox1), table, FALSE, FALSE, 0);
 	gtk_table_set_row_spacings(GTK_TABLE(table), 4);
@@ -169,7 +171,9 @@ static void prefs_template_window_create(void)
 
 	ADD_ENTRY(entry_to, _("To:"), 0);
 	address_completion_register_entry(GTK_ENTRY(entry_to));
-	ADD_ENTRY(entry_subject, _("Subject:"), 1);
+	ADD_ENTRY(entry_cc, _("Cc:"), 1);
+	address_completion_register_entry(GTK_ENTRY(entry_cc));
+	ADD_ENTRY(entry_subject, _("Subject:"), 2);
 
 #undef ADD_ENTRY
 
@@ -279,8 +283,9 @@ static void prefs_template_window_create(void)
 	templates.ok_btn = ok_btn;
 	templates.clist_tmpls = clist_tmpls;
 	templates.entry_name = entry_name;
-	templates.entry_subject = entry_subject;
 	templates.entry_to = entry_to;
+	templates.entry_cc = entry_cc;
+	templates.entry_subject = entry_subject;
 	templates.text_value = text_value;
 }
 
@@ -373,6 +378,7 @@ static void prefs_template_select_cb(GtkCList *clist, gint row, gint column,
 	tmpl_def.name = _("Template");
 	tmpl_def.subject = "";
 	tmpl_def.to = "";
+	tmpl_def.cc = "";
 	tmpl_def.value = "";
 
 	if (!(tmpl = gtk_clist_get_row_data(clist, row)))
@@ -381,6 +387,8 @@ static void prefs_template_select_cb(GtkCList *clist, gint row, gint column,
 	gtk_entry_set_text(GTK_ENTRY(templates.entry_name), tmpl->name);
 	gtk_entry_set_text(GTK_ENTRY(templates.entry_to),
 			   tmpl->to ? tmpl->to : "");
+	gtk_entry_set_text(GTK_ENTRY(templates.entry_cc),
+			   tmpl->cc ? tmpl->cc : "");
 	gtk_entry_set_text(GTK_ENTRY(templates.entry_subject),
 			   tmpl->subject ? tmpl->subject : "");
 	
@@ -415,8 +423,9 @@ static gint prefs_template_clist_set_row(gint row)
 	Template *tmpl;
 	Template *tmp_tmpl;
 	gchar *name;
-	gchar *subject;
 	gchar *to;
+	gchar *cc;
+	gchar *subject;
 	gchar *value;
 	gchar *title[1];
 
@@ -445,8 +454,8 @@ static gint prefs_template_clist_set_row(gint row)
 				      0, -1);
 	subject = gtk_editable_get_chars(GTK_EDITABLE(templates.entry_subject),
 					 0, -1);
-	to = gtk_editable_get_chars(GTK_EDITABLE(templates.entry_to),
-				    0, -1);
+	to = gtk_editable_get_chars(GTK_EDITABLE(templates.entry_to), 0, -1);
+	cc = gtk_editable_get_chars(GTK_EDITABLE(templates.entry_cc), 0, -1);
 
 	if (subject && *subject == '\0') {
 		g_free(subject);
@@ -459,8 +468,9 @@ static gint prefs_template_clist_set_row(gint row)
 
 	tmpl = g_new(Template, 1);
 	tmpl->name = name;
-	tmpl->subject = subject;
 	tmpl->to = to;
+	tmpl->cc = cc;
+	tmpl->subject = subject;
 	tmpl->value = value;
 
 	title[0] = name;
