@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2001 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2004 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@ typedef struct _MimeType	MimeType;
 typedef struct _MimeInfo	MimeInfo;
 
 #include "procmsg.h"
+#include "utils.h"
 
 typedef enum
 {
@@ -158,10 +159,10 @@ FILE *procmime_get_first_text_content	(MsgInfo	*msginfo);
 gboolean procmime_find_string_part	(MimeInfo	*mimeinfo,
 					 const gchar	*filename,
 					 const gchar	*str,
-					 gboolean	 case_sens);
+					 StrFindFunc	 find_func);
 gboolean procmime_find_string		(MsgInfo	*msginfo,
 					 const gchar	*str,
-					 gboolean	 case_sens);
+					 StrFindFunc	 find_func);
 
 gchar *procmime_get_tmp_file_name	(MimeInfo	*mimeinfo);
 
@@ -169,7 +170,7 @@ ContentType procmime_scan_mime_type	(const gchar	*mime_type);
 gchar *procmime_get_mime_type		(const gchar	*filename);
 
 EncodingType procmime_get_encoding_for_charset	(const gchar	*charset);
-EncodingType procmime_get_encoding_for_file	(const gchar	*file);
+EncodingType procmime_get_encoding_for_text_file(const gchar	*file);
 const gchar *procmime_get_encoding_str		(EncodingType	 encoding);
 
 #endif /* __PROCMIME_H__ */
