@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2004 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2005 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -45,9 +45,11 @@ static void mark_sum_func			(gpointer	 key,
 						 gpointer	 data);
 
 static GHashTable *procmsg_read_mark_file	(FolderItem	*item);
+static void procmsg_write_mark_file		(FolderItem	*item,
+						 GHashTable	*mark_table);
 
 static FILE *procmsg_open_data_file		(const gchar	*file,
-						 gint		 version,
+						 guint		 version,
 						 DataOpenMode	 mode,
 						 gchar		*buf,
 						 size_t		 buf_size);
@@ -198,7 +200,7 @@ GSList *procmsg_read_cache(FolderItem *item, gboolean scan_file)
 	MsgInfo *msginfo;
 	MsgFlags default_flags;
 	gchar file_buf[BUFFSIZE];
-	guint num;
+	guint32 num;
 	FolderType type;
 
 	g_return_val_if_fail(item != NULL, NULL);
@@ -206,7 +208,7 @@ GSList *procmsg_read_cache(FolderItem *item, gboolean scan_file)
 	type = FOLDER_TYPE(item->folder);
 
 	default_flags.perm_flags = MSG_NEW|MSG_UNREAD;
-	default_flags.tmp_flags = MSG_CACHED;
+	default_flags.tmp_flags = 0;
 	if (type == F_MH || type == F_IMAP) {
 		if (item->stype == F_QUEUE) {
 			MSG_SET_TMP_FLAGS(default_flags, MSG_QUEUED);
@@ -260,8 +262,8 @@ GSList *procmsg_read_cache(FolderItem *item, gboolean scan_file)
 
 		/* if the message file doesn't exist or is changed,
 		   don't add the data */
-		if (type == F_MH && scan_file &&
-		    folder_item_is_msg_changed(item, msginfo))
+		if ((type == F_MH && scan_file &&
+		     folder_item_is_msg_changed(item, msginfo)) || num == 0)
 			procmsg_msginfo_free(msginfo);
 		else {
 			msginfo->folder = item;
@@ -285,12 +287,18 @@ GSList *procmsg_read_cache(FolderItem *item, gboolean scan_file)
 #undef READ_CACHE_DATA
 #undef READ_CACHE_DATA_INT
 
+static void mark_unset_new_func(gpointer key, gpointer value, gpointer data)
+{
+	MSG_UNSET_PERM_FLAGS(*((MsgFlags *)value), MSG_NEW);
+}
+
 void procmsg_set_flags(GSList *mlist, FolderItem *item)
 {
-	GSList *cur, *tmp;
+	GSList *cur;
 	gint new = 0, unread = 0, total = 0;
 	gint lastnum = 0;
 	gint unflagged = 0;
+	gboolean mark_queue_exist;
 	MsgInfo *msginfo;
 	GHashTable *mark_table;
 	MsgFlags *flags;
@@ -300,6 +308,7 @@ void procmsg_set_flags(GSList *mlist, FolderItem *item)
 
 	debug_print("Marking the messages...\n");
 
+	mark_queue_exist = (item->mark_queue != NULL);
 	mark_table = procmsg_read_mark_file(item);
 	if (!mark_table) {
 		item->new = item->unread = item->total = g_slist_length(mlist);
@@ -307,6 +316,20 @@ void procmsg_set_flags(GSList *mlist, FolderItem *item)
 		return;
 	}
 
+	/* unset new flags if new (unflagged) messages exist */
+	if (!mark_queue_exist) {
+		for (cur = mlist; cur != NULL; cur = cur->next) {
+			msginfo = (MsgInfo *)cur->data;
+			flags = g_hash_table_lookup
+				(mark_table, GUINT_TO_POINTER(msginfo->msgnum));
+			if (!flags) {
+				g_hash_table_foreach(mark_table,
+						     mark_unset_new_func, NULL);
+				break;
+			}
+		}
+	}
+
 	for (cur = mlist; cur != NULL; cur = cur->next) {
 		msginfo = (MsgInfo *)cur->data;
 
@@ -329,18 +352,6 @@ void procmsg_set_flags(GSList *mlist, FolderItem *item)
 				MSG_SET_TMP_FLAGS(msginfo->flags, MSG_NEWS);
 			}
 		} else {
-			/* not found (new message) */
-			if (unflagged == 0) {
-				for (tmp = mlist; tmp != cur; tmp = tmp->next) {
-					msginfo = (MsgInfo *)tmp->data;
-					if (MSG_IS_NEW(msginfo->flags)) {
-						MSG_UNSET_PERM_FLAGS
-							(msginfo->flags,
-							 MSG_NEW);
-						--new;
-					}
-				}
-			}
 			++unflagged;
 			++new;
 			++unread;
@@ -465,6 +476,9 @@ void procmsg_flush_mark_queue(FolderItem *item, FILE *fp)
 	g_return_if_fail(item != NULL);
 	g_return_if_fail(fp != NULL);
 
+	if (item->mark_queue)
+		debug_print("flushing mark_queue...\n");
+
 	while (item->mark_queue != NULL) {
 		flaginfo = (MsgInfo *)item->mark_queue->data;
 		procmsg_write_flags(flaginfo, fp);
@@ -473,6 +487,18 @@ void procmsg_flush_mark_queue(FolderItem *item, FILE *fp)
 	}
 }
 
+void procmsg_add_mark_queue(FolderItem *item, gint num, MsgFlags flags)
+{
+	MsgInfo *queue_msginfo;
+
+	queue_msginfo = g_new0(MsgInfo, 1);
+	queue_msginfo->msgnum = num;
+	queue_msginfo->flags = flags;
+	item->mark_queue = g_slist_append
+		(item->mark_queue, queue_msginfo);
+	return;
+}
+
 void procmsg_add_flags(FolderItem *item, gint num, MsgFlags flags)
 {
 	FILE *fp;
@@ -481,13 +507,7 @@ void procmsg_add_flags(FolderItem *item, gint num, MsgFlags flags)
 	g_return_if_fail(item != NULL);
 
 	if (item->opened) {
-		MsgInfo *queue_msginfo;
-
-		queue_msginfo = g_new0(MsgInfo, 1);
-		queue_msginfo->msgnum = num;
-		queue_msginfo->flags = flags;
-		item->mark_queue = g_slist_append
-			(item->mark_queue, queue_msginfo);
+		procmsg_add_mark_queue(item, num, flags);
 		return;
 	}
 
@@ -557,17 +577,21 @@ static GHashTable *procmsg_read_mark_file(FolderItem *item)
 {
 	FILE *fp;
 	GHashTable *mark_table = NULL;
-	gint num;
+	guint32 idata;
+	guint num;
 	MsgFlags *flags;
 	MsgPermFlags perm_flags;
+	GSList *cur;
 
 	if ((fp = procmsg_open_mark_file(item, DATA_READ)) == NULL)
 		return NULL;
 
 	mark_table = g_hash_table_new(NULL, g_direct_equal);
 
-	while (fread(&num, sizeof(num), 1, fp) == 1) {
-		if (fread(&perm_flags, sizeof(perm_flags), 1, fp) != 1) break;
+	while (fread(&idata, sizeof(idata), 1, fp) == 1) {
+		num = idata;
+		if (fread(&idata, sizeof(idata), 1, fp) != 1) break;
+		perm_flags = idata;
 
 		flags = g_hash_table_lookup(mark_table, GUINT_TO_POINTER(num));
 		if (flags != NULL)
@@ -580,15 +604,60 @@ static GHashTable *procmsg_read_mark_file(FolderItem *item)
 	}
 
 	fclose(fp);
+
+	if (item->mark_queue) {
+		g_hash_table_foreach(mark_table, mark_unset_new_func, NULL);
+	}
+
+	for (cur = item->mark_queue; cur != NULL; cur = cur->next) {
+		MsgInfo *msginfo = (MsgInfo *)cur->data;
+
+		flags = g_hash_table_lookup(mark_table,
+					    GUINT_TO_POINTER(msginfo->msgnum));
+		if (flags != NULL)
+			g_free(flags);
+
+		flags = g_new0(MsgFlags, 1);
+		flags->perm_flags = msginfo->flags.perm_flags;
+
+		g_hash_table_insert(mark_table,
+				    GUINT_TO_POINTER(msginfo->msgnum), flags);
+				    
+	}
+
+	if (item->mark_queue && !item->opened) {
+		procmsg_write_mark_file(item, mark_table);
+		procmsg_msg_list_free(item->mark_queue);
+		item->mark_queue = NULL;
+	}
+
 	return mark_table;
 }
 
-static FILE *procmsg_open_data_file(const gchar *file, gint version,
+static void write_mark_func(gpointer key, gpointer value, gpointer data)
+{
+	MsgInfo msginfo;
+
+	msginfo.msgnum = GPOINTER_TO_UINT(key);
+	msginfo.flags.perm_flags = ((MsgFlags *)value)->perm_flags;
+	procmsg_write_flags(&msginfo, (FILE *)data);
+}
+
+static void procmsg_write_mark_file(FolderItem *item, GHashTable *mark_table)
+{
+	FILE *fp;
+
+	fp = procmsg_open_mark_file(item, DATA_WRITE);
+	g_hash_table_foreach(mark_table, write_mark_func, fp);
+	fclose(fp);
+}
+
+static FILE *procmsg_open_data_file(const gchar *file, guint version,
 				    DataOpenMode mode,
 				    gchar *buf, size_t buf_size)
 {
 	FILE *fp;
-	gint data_ver;
+	guint32 data_ver;
 
 	g_return_val_if_fail(file != NULL, NULL);
 
@@ -611,8 +680,8 @@ static FILE *procmsg_open_data_file(const gchar *file, gint version,
 		if (buf && buf_size > 0)
 			setvbuf(fp, buf, _IOFBF, buf_size);
 		if (fread(&data_ver, sizeof(data_ver), 1, fp) != 1 ||
-			 version != data_ver) {
-			debug_print("Mark/Cache version is different (%d != %d). "
+		    version != data_ver) {
+			debug_print("Mark/Cache version is different (%u != %u). "
 				    "Discarding it.\n", data_ver, version);
 			fclose(fp);
 			fp = NULL;
@@ -815,6 +884,8 @@ gchar *procmsg_get_message_file_path(MsgInfo *msginfo)
 
 	if (msginfo->plaintext_file)
 		file = g_strdup(msginfo->plaintext_file);
+	else if (msginfo->file_path)
+		return g_strdup(msginfo->file_path);
 	else {
 		path = folder_item_get_path(msginfo->folder);
 		file = g_strconcat(path, G_DIR_SEPARATOR_S,
@@ -831,6 +902,9 @@ gchar *procmsg_get_message_file(MsgInfo *msginfo)
 
 	g_return_val_if_fail(msginfo != NULL, NULL);
 
+	if (msginfo->file_path)
+		return g_strdup(msginfo->file_path);
+
 	filename = folder_item_fetch_msg(msginfo->folder, msginfo->msgnum);
 	if (!filename)
 		debug_print(_("can't fetch message %d\n"), msginfo->msgnum);
@@ -1081,15 +1155,29 @@ void procmsg_get_filter_keyword(MsgInfo *msginfo, gchar **header, gchar **key,
 	}
 }
 
-void procmsg_empty_trash(void)
+void procmsg_empty_trash(FolderItem *trash)
+{
+	FILE *fp;
+
+	if (trash && trash->total > 0) {
+		debug_print("Emptying messages in %s ...\n", trash->path);
+
+		folder_item_remove_all_msg(trash);
+		fp = procmsg_open_cache_file(trash, DATA_WRITE);
+		if (fp) fclose(fp);
+		fp = procmsg_open_mark_file(trash, DATA_WRITE);
+		if (fp) fclose(fp);
+	}
+}
+
+void procmsg_empty_all_trash(void)
 {
 	FolderItem *trash;
 	GList *cur;
 
 	for (cur = folder_get_list(); cur != NULL; cur = cur->next) {
 		trash = FOLDER(cur->data)->trash;
-		if (trash && trash->total > 0)
-			folder_item_remove_all_msg(trash);
+		procmsg_empty_trash(trash);
 	}
 }
 
@@ -1115,7 +1203,7 @@ gint procmsg_send_queue(FolderItem *queue, gboolean save_msgs)
 			QueueInfo *qinfo;
 
 			qinfo = send_get_queue_info(file);
-			if (send_message_queue(qinfo) < 0) {
+			if (!qinfo || send_message_queue(qinfo) < 0) {
 				g_warning(_("Sending queued message %d failed.\n"),
 					  msginfo->msgnum);
 			} else {
@@ -1271,6 +1359,7 @@ MsgInfo *procmsg_msginfo_copy(MsgInfo *msginfo)
 	MEMBCOPY(size);
 	MEMBCOPY(mtime);
 	MEMBCOPY(date_t);
+
 	MEMBCOPY(flags);
 
 	MEMBDUP(fromname);
@@ -1289,6 +1378,11 @@ MsgInfo *procmsg_msginfo_copy(MsgInfo *msginfo)
 
 	MEMBDUP(xface);
 
+	MEMBDUP(file_path);
+
+	MEMBDUP(plaintext_file);
+	MEMBCOPY(decryption_failed);
+
 	return newmsginfo;
 }
 
@@ -1314,6 +1408,9 @@ MsgInfo *procmsg_msginfo_get_full_info(MsgInfo *msginfo)
 	full_msginfo->mtime = msginfo->mtime;
 	full_msginfo->folder = msginfo->folder;
 	full_msginfo->to_folder = msginfo->to_folder;
+
+	full_msginfo->file_path = g_strdup(msginfo->file_path);
+
 #if USE_GPGME
 	full_msginfo->plaintext_file = g_strdup(msginfo->plaintext_file);
 	full_msginfo->decryption_failed = msginfo->decryption_failed;
@@ -1339,6 +1436,10 @@ void procmsg_msginfo_free(MsgInfo *msginfo)
 	g_free(msginfo->msgid);
 	g_free(msginfo->inreplyto);
 
+	g_free(msginfo->file_path);
+
+	g_free(msginfo->plaintext_file);
+
 	g_free(msginfo);
 }
 
