@@ -251,7 +251,7 @@ int quote_fmtwrap(void)
 	return 1;
 }
 
-static int isseparator(char ch)
+static int isseparator(int ch)
 {
 	return isspace(ch) || ch == '.' || ch == '-';
 }
@@ -1269,7 +1269,7 @@ yyreduce:
 #line 195 "quote_fmt_parse.y"
     {
 		if (msginfo->fromname) {
-			gchar *p;
+			guchar *p;
 			gchar *str;
 
 			str = alloca(strlen(msginfo->fromname) + 1);
@@ -1290,7 +1290,7 @@ yyreduce:
 #define MAX_SENDER_INITIAL 20
 		if (msginfo->fromname) {
 			gchar tmp[MAX_SENDER_INITIAL];
-			gchar *p;
+			guchar *p;
 			gchar *cur;
 			gint len = 0;
 
