@@ -40,6 +40,12 @@ typedef enum {
 	SSL_METHOD_TLSv1
 } SSLMethod;
 
+typedef enum {
+	SSL_NONE,
+	SSL_TUNNEL,
+	SSL_STARTTLS
+} SSLType;
+
 void ssl_init				(void);
 void ssl_done				(void);
 gboolean ssl_init_socket		(SockInfo	*sockinfo);
