@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2003 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2004 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -71,6 +71,13 @@
 #include "pixmaps/vcard.xpm"
 #include "pixmaps/online.xpm"
 #include "pixmaps/offline.xpm"
+#include "pixmaps/stock_dialog_info_48.xpm"
+#include "pixmaps/stock_dialog_question_48.xpm"
+#include "pixmaps/stock_dialog_warning_48.xpm"
+#include "pixmaps/stock_dialog_error_48.xpm"
+#include "pixmaps/stock_add_16.xpm"
+#include "pixmaps/stock_remove_16.xpm"
+#include "pixmaps/mail.xpm"
 
 typedef struct _StockPixmapData	StockPixmapData;
 
@@ -130,6 +137,13 @@ static StockPixmapData pixmaps[] =
 	{vcard_xpm			, NULL, NULL},
 	{online_xpm			, NULL, NULL},
 	{offline_xpm			, NULL, NULL},
+	{stock_dialog_info_48_xpm	, NULL, NULL},
+	{stock_dialog_question_48_xpm	, NULL, NULL},
+	{stock_dialog_warning_48_xpm	, NULL, NULL},
+	{stock_dialog_error_48_xpm	, NULL, NULL},
+	{stock_add_16_xpm		, NULL, NULL},
+	{stock_remove_16_xpm		, NULL, NULL},
+	{mail_xpm			, NULL, NULL},
 };
 
 /* return newly constructed GtkPixmap from GdkPixmap */
