@@ -1,7 +1,7 @@
 /*
  * Sylpheed templates subsystem 
  * Copyright (C) 2001 Alexander Barinov
- * Copyright (C) 2001 Hiroyuki Yamamoto
+ * Copyright (C) 2001-2004 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -47,8 +47,9 @@ static Template *template_load(gchar *filename)
 
 	tmpl = g_new(Template, 1);
 	tmpl->name = NULL;
-	tmpl->subject = NULL;
 	tmpl->to = NULL;
+	tmpl->cc = NULL;
+	tmpl->subject = NULL;
 	tmpl->value = NULL;
 
 	while (fgets(buf, sizeof(buf), fp) != NULL) {
@@ -56,10 +57,12 @@ static Template *template_load(gchar *filename)
 			break;
 		else if (!g_strncasecmp(buf, "Name:", 5))
 			tmpl->name = g_strdup(g_strstrip(buf + 5));
-		else if (!g_strncasecmp(buf, "Subject:", 8))
-			tmpl->subject = g_strdup(g_strstrip(buf + 8));
 		else if (!g_strncasecmp(buf, "To:", 3))
 			tmpl->to = g_strdup(g_strstrip(buf + 3));
+		else if (!g_strncasecmp(buf, "Cc:", 3))
+			tmpl->cc = g_strdup(g_strstrip(buf + 3));
+		else if (!g_strncasecmp(buf, "Subject:", 8))
+			tmpl->subject = g_strdup(g_strstrip(buf + 8));
 	}
 
 	if (!tmpl->name) {
@@ -84,8 +87,9 @@ static Template *template_load(gchar *filename)
 void template_free(Template *tmpl)
 {
 	g_free(tmpl->name);
-	g_free(tmpl->subject);
 	g_free(tmpl->to);
+	g_free(tmpl->cc);
+	g_free(tmpl->subject);
 	g_free(tmpl->value);
 	g_free(tmpl);
 }
@@ -183,18 +187,20 @@ void template_write_config(GSList *tmpl_list)
 		if ((fp = fopen(filename, "wb")) == NULL) {
 			FILE_OP_ERROR(filename, "fopen");
 			g_free(filename);
-			g_free(path);
 			return;
 		}
 
 		fprintf(fp, "Name: %s\n", tmpl->name);
-		if (tmpl->subject && *tmpl->subject != '\0')
-			fprintf(fp, "Subject: %s\n", tmpl->subject);
 		if (tmpl->to && *tmpl->to != '\0')
 			fprintf(fp, "To: %s\n", tmpl->to);
+		if (tmpl->cc && *tmpl->cc != '\0')
+			fprintf(fp, "Cc: %s\n", tmpl->cc);
+		if (tmpl->subject && *tmpl->subject != '\0')
+			fprintf(fp, "Subject: %s\n", tmpl->subject);
 		fputs("\n", fp);
 		fwrite(tmpl->value, sizeof(gchar) * strlen(tmpl->value), 1, fp);
 		fclose(fp);
+		g_free(filename);
 	}
 }
 
