@@ -182,7 +182,6 @@ static void undo_check_size(UndoMain *undostruct)
 		undostruct->undo = g_list_remove(undostruct->undo, last_undo);
 		undo_object_free(last_undo);
 	}
-	debug_print("g_list_length(undostruct->undo): %d\n", length);
 }
 
 /**
@@ -280,7 +279,6 @@ static gint undo_merge(GList *list, guint start_pos, guint end_pos,
 	} else
 		debug_print("Unknown action [%i] inside undo merge encountered", action);
 
-	debug_print("Merged: %s\n", text);
 	return TRUE;
 }
 
@@ -303,8 +301,6 @@ static void undo_add(const gchar *text,
 {
 	UndoInfo *undoinfo;
 
-	debug_print("undo_add(%i)*%s*\n", strlen (text), text);
-
 	g_return_if_fail(text != NULL);
 	g_return_if_fail(end_pos >= start_pos);
 
@@ -330,8 +326,6 @@ static void undo_add(const gchar *text,
 
 	undo_check_size(undostruct);
 
-	debug_print("New: %s Action: %d Paste: %d\n", text, action, undostruct->paste);
-
 	undoinfo = undo_object_new(g_strdup(text), start_pos, end_pos, action,
 				   GTK_ADJUSTMENT(GTK_STEXT(undostruct->text)->vadj)->value);
 
