@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2003 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2004 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -125,6 +125,26 @@ void ptr_array_free_strings(GPtrArray *array)
 	}
 }
 
+gboolean str_find(const gchar *haystack, const gchar *needle)
+{
+	return strstr(haystack, needle) != NULL ? TRUE : FALSE;
+}
+
+gboolean str_case_find(const gchar *haystack, const gchar *needle)
+{
+	return strcasestr(haystack, needle) != NULL ? TRUE : FALSE;
+}
+
+gboolean str_find_equal(const gchar *haystack, const gchar *needle)
+{
+	return strcmp(haystack, needle) == 0;
+}
+
+gboolean str_case_find_equal(const gchar *haystack, const gchar *needle)
+{
+	return strcasecmp(haystack, needle) == 0;
+}
+
 gint to_number(const gchar *nstr)
 {
 	register const guchar *p;
@@ -263,6 +283,29 @@ gchar *strcasestr(const gchar *haystack, const gchar *needle)
 	return NULL;
 }
 
+gpointer my_memmem(gconstpointer haystack, size_t haystacklen,
+		   gconstpointer needle, size_t needlelen)
+{
+	const gchar *haystack_ = (const gchar *)haystack;
+	const gchar *needle_ = (const gchar *)needle;
+	const gchar *haystack_cur = (const gchar *)haystack;
+
+	if (needlelen == 1)
+		return memchr(haystack_, *needle_, haystacklen);
+
+	while ((haystack_cur = memchr(haystack_cur, *needle_, haystacklen))
+	       != NULL) {
+		if (haystacklen - (haystack_cur - haystack_) < needlelen)
+			break;
+		if (memcmp(haystack_cur + 1, needle_ + 1, needlelen - 1) == 0)
+			return (gpointer)haystack_cur;
+		else
+			haystack_cur++;
+	}
+
+	return NULL;
+}
+
 /* Copy no more than N characters of SRC to DEST, with NULL terminating.  */
 gchar *strncpy2(gchar *dest, const gchar *src, size_t n)
 {
@@ -1004,7 +1047,7 @@ void subst_chars(gchar *str, gchar *orig, gchar subst)
 
 void subst_for_filename(gchar *str)
 {
-	subst_chars(str, " \t\r\n\"/\\", '_');
+	subst_chars(str, " \t\r\n\"'/\\", '_');
 }
 
 gboolean is_header_line(const gchar *str)
@@ -1306,8 +1349,8 @@ gchar *trim_string(const gchar *str, gint len)
 			return g_strdup(str);
 		else if (new_len + mb_len > len)
 			break;
-		else
-			new_len += mb_len;
+
+		new_len += mb_len;
 		p += mb_len;
 	}
 
@@ -1315,6 +1358,33 @@ gchar *trim_string(const gchar *str, gint len)
 	return g_strconcat(new_str, "...", NULL);
 }
 
+gchar *trim_string_before(const gchar *str, gint len)
+{
+	const gchar *p = str;
+	gint mb_len;
+	gint new_len;
+
+	if (!str) return NULL;
+	if ((new_len = strlen(str)) <= len)
+		return g_strdup(str);
+
+	while (*p != '\0') {
+		mb_len = mblen(p, MB_LEN_MAX);
+		if (mb_len == 0)
+			break;
+		else if (mb_len < 0)
+			return g_strdup(str);
+
+		new_len -= mb_len;
+		p += mb_len;
+
+		if (new_len <= len)
+			break;
+	}
+
+	return g_strconcat("...", p, NULL);
+}
+
 GList *uri_list_extract_filenames(const gchar *uri_list)
 {
 	GList *result = NULL;
@@ -1871,7 +1941,7 @@ gint remove_numbered_files(const gchar *dir, guint first, guint last)
 	DIR *dp;
 	struct dirent *d;
 	gchar *prev_dir;
-	gint fileno;
+	gint file_no;
 
 	prev_dir = g_get_current_dir();
 
@@ -1888,8 +1958,8 @@ gint remove_numbered_files(const gchar *dir, guint first, guint last)
 	}
 
 	while ((d = readdir(dp)) != NULL) {
-		fileno = to_number(d->d_name);
-		if (fileno >= 0 && first <= fileno && fileno <= last) {
+		file_no = to_number(d->d_name);
+		if (file_no > 0 && first <= file_no && file_no <= last) {
 			if (is_dir_exist(d->d_name))
 				continue;
 			if (unlink(d->d_name) < 0)
@@ -1921,7 +1991,7 @@ gint remove_expired_files(const gchar *dir, guint hours)
 	struct dirent *d;
 	struct stat s;
 	gchar *prev_dir;
-	gint fileno;
+	gint file_no;
 	time_t mtime, now, expire_time;
 
 	prev_dir = g_get_current_dir();
@@ -1942,8 +2012,8 @@ gint remove_expired_files(const gchar *dir, guint hours)
 	expire_time = hours * 60 * 60;
 
 	while ((d = readdir(dp)) != NULL) {
-		fileno = to_number(d->d_name);
-		if (fileno >= 0) {
+		file_no = to_number(d->d_name);
+		if (file_no > 0) {
 			if (stat(d->d_name, &s) < 0) {
 				FILE_OP_ERROR(d->d_name, "stat");
 				continue;
@@ -2707,6 +2777,7 @@ gchar *file_read_stream_to_str(FILE *fp)
 gint execute_async(gchar *const argv[])
 {
 	pid_t pid;
+	gint status;
 
 	if ((pid = fork()) < 0) {
 		perror("fork");
@@ -2731,14 +2802,18 @@ gint execute_async(gchar *const argv[])
 		_exit(0);
 	}
 
-	waitpid(pid, NULL, 0);
+	waitpid(pid, &status, 0);
 
-	return 0;
+	if (WIFEXITED(status))
+		return WEXITSTATUS(status);
+	else
+		return -1;
 }
 
 gint execute_sync(gchar *const argv[])
 {
 	pid_t pid;
+	gint status;
 
 	if ((pid = fork()) < 0) {
 		perror("fork");
@@ -2752,9 +2827,12 @@ gint execute_sync(gchar *const argv[])
 		_exit(1);
 	}
 
-	waitpid(pid, NULL, 0);
+	waitpid(pid, &status, 0);
 
-	return 0;
+	if (WIFEXITED(status))
+		return WEXITSTATUS(status);
+	else
+		return -1;
 }
 
 gint execute_command_line(const gchar *cmdline, gboolean async)
@@ -2762,12 +2840,15 @@ gint execute_command_line(const gchar *cmdline, gboolean async)
 	gchar **argv;
 	gint ret;
 
+	debug_print("executing: %s\n", cmdline);
+
 	argv = strsplit_with_quote(cmdline, " ", 0);
 
 	if (async)
 		ret = execute_async(argv);
 	else
 		ret = execute_sync(argv);
+
 	g_strfreev(argv);
 
 	return ret;
@@ -2813,7 +2894,8 @@ gint open_uri(const gchar *uri, const gchar *cmdline)
 		g_snprintf(buf, sizeof(buf), cmdline, uri);
 	else {
 		if (cmdline)
-			g_warning(_("Open URI command line is invalid: `%s'"),
+			g_warning("Open URI command line is invalid "
+				  "(there must be only one '%%s'): %s",
 				  cmdline);
 		g_snprintf(buf, sizeof(buf), DEFAULT_BROWSER_CMD, uri);
 	}
