@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2003 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2004 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -160,6 +160,18 @@ guint str_case_hash		(gconstpointer	 key);
 
 void ptr_array_free_strings	(GPtrArray	*array);
 
+typedef gboolean (*StrFindFunc) (const gchar	*haystack,
+				 const gchar	*needle);
+
+gboolean str_find		(const gchar	*haystack,
+				 const gchar	*needle);
+gboolean str_case_find		(const gchar	*haystack,
+				 const gchar	*needle);
+gboolean str_find_equal		(const gchar	*haystack,
+				 const gchar	*needle);
+gboolean str_case_find_equal	(const gchar	*haystack,
+				 const gchar	*needle);
+
 /* number-string conversion */
 gint to_number			(const gchar *nstr);
 gchar *itos_buf			(gchar	     *nstr,
@@ -178,6 +190,10 @@ gchar *strtailchomp	(gchar		*str,
 gchar *strcrchomp	(gchar		*str);
 gchar *strcasestr	(const gchar	*haystack,
 			 const gchar	*needle);
+gpointer my_memmem	(gconstpointer	 haystack,
+			 size_t		 haystacklen,
+			 gconstpointer	 needle,
+			 size_t		 needlelen);
 gchar *strncpy2		(gchar		*dest,
 			 const gchar	*src,
 			 size_t		 n);
@@ -297,6 +313,8 @@ gchar *get_abbrev_newsgroup_name	(const gchar	*group,
 					 gint		 len);
 gchar *trim_string			(const gchar	*str,
 					 gint		 len);
+gchar *trim_string_before		(const gchar	*str,
+					 gint		 len);
 
 GList *uri_list_extract_filenames	(const gchar	*uri_list);
 gboolean is_uri_string			(const gchar	*str);
