@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2001 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2004 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 #include "xml.h"
 #include "main.h"
 #include "utils.h"
+#include "codeconv.h"
 
 #define SPARSE_MEMORY
 /* if this is defined all attr.names and tag.names are stored
@@ -121,8 +122,7 @@ static GNode *xml_build_tree(XMLFile *file, GNode *parent, guint level)
 
 		tag = xml_get_current_tag(file);
 		if (!tag) break;
-		xmlnode = g_new(XMLNode, 1);
-		xmlnode->tag = xml_copy_tag(tag);
+		xmlnode = xml_node_new(xml_copy_tag(tag), NULL);
 		xmlnode->element = xml_get_element(file);
 		if (!parent)
 			node = g_node_new(xmlnode);
@@ -206,7 +206,7 @@ gint xml_parse_next_tag(XMLFile *file)
 		return 0;
 	}
 
-	tag = g_new0(XMLTag, 1);
+	tag = xml_tag_new(NULL);
 	xml_push_tag(file, tag);
 
 	len = strlen(buf);
@@ -264,10 +264,8 @@ gint xml_parse_next_tag(XMLFile *file)
 		g_strchomp(attr_name);
 		xml_unescape_str(attr_value);
 
-		attr = g_new(XMLAttr, 1);
-		attr->name  = XML_STRING_ADD(attr_name);
-		attr->value = g_strdup(attr_value);
-		tag->attr   = g_list_append(tag->attr, attr);
+		attr = xml_attr_new(attr_name, attr_value);
+		xml_tag_add_attr(tag, attr);
 	}
 
 	return 0;
@@ -379,18 +377,57 @@ gboolean xml_compare_tag(XMLFile *file, const gchar *name)
 		return FALSE;
 }
 
+XMLNode *xml_node_new(XMLTag *tag, const gchar *text)
+{
+	XMLNode *node;
+
+	node = g_new(XMLNode, 1);
+	node->tag = tag;
+	node->element = g_strdup(text);
+
+	return node;
+}
+
+XMLTag *xml_tag_new(const gchar *tag)
+{
+	XMLTag *new_tag;
+
+	new_tag = g_new(XMLTag, 1);
+	if (tag)
+		new_tag->tag = XML_STRING_ADD(tag);
+	else
+		new_tag->tag = NULL;
+	new_tag->attr = NULL;
+
+	return new_tag;
+}
+
+XMLAttr *xml_attr_new(const gchar *name, const gchar *value)
+{
+	XMLAttr *new_attr;
+
+	new_attr = g_new(XMLAttr, 1);
+	new_attr->name = XML_STRING_ADD(name);
+	new_attr->value = g_strdup(value);
+
+	return new_attr;
+}
+
+void xml_tag_add_attr(XMLTag *tag, XMLAttr *attr)
+{
+	tag->attr = g_list_append(tag->attr, attr);
+}
+
 XMLTag *xml_copy_tag(XMLTag *tag)
 {
 	XMLTag *new_tag;
 	XMLAttr *attr;
 	GList *list;
 
-	new_tag = g_new(XMLTag, 1);
-	new_tag->tag = XML_STRING_ADD(tag->tag);
-	new_tag->attr = NULL;
+	new_tag = xml_tag_new(tag->tag);
 	for (list = tag->attr; list != NULL; list = list->next) {
 		attr = xml_copy_attr((XMLAttr *)list->data);
-		new_tag->attr = g_list_append(new_tag->attr, attr);
+		xml_tag_add_attr(new_tag, attr);
 	}
 
 	return new_tag;
@@ -398,13 +435,7 @@ XMLTag *xml_copy_tag(XMLTag *tag)
 
 XMLAttr *xml_copy_attr(XMLAttr *attr)
 {
-	XMLAttr *new_attr;
-
-	new_attr = g_new(XMLAttr, 1);
-	new_attr->name  = XML_STRING_ADD(attr->name);
-	new_attr->value = g_strdup(attr->value);
-
-	return new_attr;
+	return xml_attr_new(attr->name, attr->value);
 }
 
 gint xml_unescape_str(gchar *str)
@@ -485,6 +516,42 @@ gint xml_file_put_escape_str(FILE *fp, const gchar *str)
 	return 0;
 }
 
+gint xml_file_put_xml_decl(FILE *fp)
+{
+	g_return_val_if_fail(fp != NULL, -1);
+
+	fprintf(fp, "<?xml version=\"1.0\" encoding=\"%s\"?>\n",
+		conv_get_current_charset_str());
+	return 0;
+}
+
+gint xml_file_put_node(FILE *fp, XMLNode *node)
+{
+	GList *cur;
+
+	g_return_val_if_fail(fp != NULL, -1);
+	g_return_val_if_fail(node != NULL, -1);
+
+	fprintf(fp, "<%s", node->tag->tag);
+
+	for (cur = node->tag->attr; cur != NULL; cur = cur->next) {
+		XMLAttr *attr = (XMLAttr *)cur->data;
+		fprintf(fp, " %s=\"", attr->name);
+		xml_file_put_escape_str(fp, attr->value);
+		fputs("\"", fp);
+	}
+
+	if (node->element) {
+		fputs(">", fp);
+		xml_file_put_escape_str(fp, node->element);
+		fprintf(fp, "</%s>\n", node->tag->tag);
+	} else {
+		fputs(" />\n", fp);
+	}
+
+	return 0;
+}
+
 void xml_free_node(XMLNode *node)
 {
 	if (!node) return;
